#!/usr/bin/env python

import json

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
)


__all__ = ['icourses_download', 'icourses_download_playlist']
site_info = '爱课程 icourses.cn'


def _download(item, **kwargs):
    url = item['fullLinkUrl']
    title = item['title'].strip()
    _, ext, size = url_info(url)
    print_info(site_info=site_info, title=title, type=ext, size=size)
    if not kwargs.get('info_only'):
        download_urls([url], title, ext, size, **kwargs)


def icourses_download(url, **kwargs):
    page = get_content(url)
    data = json.loads(match1(page, r'var _sourceArrStr = (.+?);'))

    if 'resId' in url:  # 下载播放列表中指定视频
        _id = match1(url, r'resId=([\w-]+)')
        results = list(filter(lambda x: x['id'] == _id, data))
        _download(results[0], **kwargs)
    else:  # 下载第一个
        _download(data[0], **kwargs)


def icourses_download_playlist(url, **kwargs):
    page = get_content(url)
    data = json.loads(match1(page, r'var _sourceArrStr = (.+?);'))
    for item in data:
        _download(item, **kwargs)


download = icourses_download
download_playlist = icourses_download_playlist



#!/usr/bin/env python

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)


__all__ = ['alive_download']
site_info = 'Alive alive.in.th'


def alive_download(url, output_dir='.', merge=True, info_only=False, **kwargs):
    html = get_content(url)

    title = match1(html, r'<meta property="og:title" content="([^"]+)"')

    url = match1(html, r'file: "(http://alive[^"]+)"')
    _type, ext, size = url_info(url)

    print_info(site_info, title, _type, size)
    if not info_only:
        download_urls([url], title, ext, size, output_dir, merge=merge)


download = alive_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)


__all__ = ['baomihua_download', 'baomihua_download_by_id']
site_info = '爆米花 baomihua.com'


def baomihua_download_by_id(
    _id, title=None, output_dir='.', merge=True, info_only=False, **kwargs
):
    html = get_content(
        'http://play.baomihua.com/getvideourl.aspx?flvid={}&devicetype='
        'phone_app'.format(_id)
    )
    host = match1(html, r'host=([^&]*)')
    assert host
    _type = match1(html, r'videofiletype=([^&]*)')
    assert _type
    vid = match1(html, r'&stream_name=([^&]*)')
    assert vid
    dir_str = match1(html, r'&dir=([^&]*)').strip()
    url = 'http://{}/{}/{}.{}'.format(host, dir_str, vid, _type)
    _, ext, size = url_info(url)
    print_info(site_info, title, _type, size)
    if not info_only:
        download_urls(
            [url], title, ext, size, output_dir, merge=merge, **kwargs
        )


def baomihua_download(
    url, output_dir='.', merge=True, info_only=False, **kwargs
):
    html = get_content(url)
    title = match1(html, r'<title>(.*)</title>')
    assert title
    _id = match1(html, r'flvid\s*=\s*(\d+)')
    assert _id
    baomihua_download_by_id(
        _id, title, output_dir=output_dir, merge=merge, info_only=info_only,
        **kwargs
    )


download = baomihua_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import re

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)
from lulu.util.parser import get_parser

__all__ = ['ehow_download']
site_info = 'eHow ehow.com'


def ehow_download(url, info_only=False, **kwargs):

    assert re.search(
        r'https?://www.ehow.com/video_', url
    ), 'URL you entered is not supported'

    html = get_content(url)
    parser = get_parser(html)
    title = parser.find('meta', property='og:title')['content']
    video = parser.find('meta', property='og:video')['content']
    url = match1(video, r'source=(.+?)&')
    _type, ext, size = url_info(url)
    print_info(site_info, title, _type, size)

    if not info_only:
        download_urls([url], title, ext, size, **kwargs)


download = ehow_download
download_playlist = playlist_not_supported(site_info)



# coding=utf-8

import urllib.parse

from lulu.extractors import iqiyi
from lulu.extractors import bokecc
from lulu.extractors.netease import netease_download
from lulu.extractors.bilibili import bilibili_download
from lulu.extractors.vimeo import vimeo_download_by_id
from lulu.extractors.iqiyi import iqiyi_download_by_vid
from lulu.extractors.youku import youku_download_by_vid
from lulu.extractors.dailymotion import dailymotion_download
from lulu.extractors.yinyuetai import yinyuetai_download_by_id
from lulu.common import (
    match1,
    matchall,
    get_content,
    playlist_not_supported,
)


__all__ = ['embed_download']
site_info = 'any.any'


# refer to http://open.youku.com/tools
youku_embed_patterns = [
    'youku\.com/v_show/id_([a-zA-Z0-9=]+)',
    'player\.youku\.com/player\.php/sid/([a-zA-Z0-9=]+)/v\.swf',
    'loader\.swf\?VideoIDS=([a-zA-Z0-9=]+)',
    'player\.youku\.com/embed/([a-zA-Z0-9=]+)',
    'YKU.Player\(\'[a-zA-Z0-9]+\',{ client_id: \'[a-zA-Z0-9]+\', vid: \'([a-zA-Z0-9]+)\'',  # noqa
]


# http://www.tudou.com/programs/view/html5embed.action?type=0&amp;code=3LS_URGvl54&amp;lcode=&amp;resourceId=0_06_05_99  # noqa
tudou_embed_patterns = [
    'tudou\.com[a-zA-Z0-9\/\?=\&\.\;]+code=([a-zA-Z0-9_-]+)\&',
    'www\.tudou\.com/v/([a-zA-Z0-9_-]+)/[^"]*v\.swf',
]


# refer to http://open.tudou.com/wiki/video/info
tudou_api_patterns = []

yinyuetai_embed_patterns = ['player\.yinyuetai\.com/video/swf/(\d+)']

iqiyi_embed_patterns = [
    'player\.video\.qiyi\.com/([^/]+)/[^/]+/[^/]+/[^/]+\.swf[^"]+tvId=(\d+)'
]

netease_embed_patterns = ['(http://\w+\.163\.com/movie/[^\'"]+)']

vimeo_embed_patters = ['player\.vimeo\.com/video/(\d+)']

dailymotion_embed_patterns = ['www\.dailymotion\.com/embed/video/(\w+)']

# check the share button on http://www.bilibili.com/video/av5079467/
bilibili_embed_patterns = ['static\.hdslb\.com/miniloader\.swf.*aid=(\d+)']

# http://open.iqiyi.com/lib/player.html
iqiyi_patterns = [
    r'(?:\"|\')(https?://dispatcher\.video\.qiyi\.com\/disp\/shareplayer\.swf\?.+?)(?:\"|\')',  # noqa
    r'(?:\"|\')(https?://open\.iqiyi\.com\/developer\/player_js\/coopPlayerIndex\.html\?.+?)(?:\"|\')',  # noqa
]

bokecc_patterns = [
    r'bokecc\.com/flash/pocle/player\.swf\?siteid=(.+?)&vid=(.{32})'
]

recur_limit = 3


def embed_download(url, output_dir='.', merge=True, info_only=False, **kwargs):
    content = get_content(url)
    found = False
    title = match1(content, '<title>([^<>]+)</title>')

    vids = matchall(content, youku_embed_patterns)
    for vid in set(vids):
        found = True
        youku_download_by_vid(
            vid, title=title, output_dir=output_dir, merge=merge,
            info_only=info_only
        )

    vids = matchall(content, yinyuetai_embed_patterns)
    for vid in vids:
        found = True
        yinyuetai_download_by_id(
            vid, title=title, output_dir=output_dir, merge=merge,
            info_only=info_only
        )

    vids = matchall(content, iqiyi_embed_patterns)
    for vid in vids:
        found = True
        iqiyi_download_by_vid(
            (vid[1], vid[0]), title=title, output_dir=output_dir, merge=merge,
            info_only=info_only
        )

    urls = matchall(content, netease_embed_patterns)
    for url in urls:
        found = True
        netease_download(
            url, output_dir=output_dir, merge=merge, info_only=info_only
        )

    urls = matchall(content, vimeo_embed_patters)
    for url in urls:
        found = True
        vimeo_download_by_id(
            url, title=title, output_dir=output_dir, merge=merge,
            info_only=info_only, referer=url
        )

    urls = matchall(content, dailymotion_embed_patterns)
    for url in urls:
        found = True
        dailymotion_download(
            url, output_dir=output_dir, merge=merge, info_only=info_only
        )

    aids = matchall(content, bilibili_embed_patterns)
    for aid in aids:
        found = True
        url = 'http://www.bilibili.com/video/av{}/'.format(aid)
        bilibili_download(
            url, output_dir=output_dir, merge=merge, info_only=info_only
        )

    iqiyi_urls = matchall(content, iqiyi_patterns)
    for url in iqiyi_urls:
        found = True
        iqiyi.download(
            url, output_dir=output_dir, merge=merge, info_only=info_only,
            **kwargs
        )

    bokecc_metas = matchall(content, bokecc_patterns)
    for meta in bokecc_metas:
        found = True
        bokecc.bokecc_download_by_id(
            meta[1], output_dir=output_dir, merge=merge, info_only=info_only,
            **kwargs
        )

    if found:
        return True

    # Try harder, check all iframes
    if 'recur_lv' not in kwargs or kwargs['recur_lv'] < recur_limit:
        r = kwargs.get('recur_lv')
        if r is None:
            r = 1
        else:
            r += 1
        iframes = matchall(content, [r'<iframe.+?src=(?:\"|\')(.+?)(?:\"|\')'])
        for iframe in iframes:
            if not iframe.startswith('http'):
                src = urllib.parse.urljoin(url, iframe)
            else:
                src = iframe
            found = embed_download(
                src, output_dir=output_dir, merge=merge, info_only=info_only,
                recur_lv=r, **kwargs
            )
            if found:
                return True

    if not found and 'recur_lv' not in kwargs:
        raise NotImplementedError(url)
    else:
        return found


download = embed_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

from html import unescape

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)


__all__ = ['ifeng_download', 'ifeng_download_by_id']
site_info = '凤凰网 ifeng.com'


def ifeng_download_by_id(_id, title=None, info_only=False, **kwargs):
    assert match1(
        _id, r'([0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12})'
    ), _id
    url = 'http://vxml.ifengimg.com/video_info_new/{}/{}/{}.xml'.format(
        _id[-2], _id[-2:], _id
    )
    xml = get_content(url)
    title = match1(xml, r'Name="([^"]+)"')
    title = unescape(title)
    url = match1(xml, r'VideoPlayUrl="([^"]+)"')
    url = url.replace(
        'http://wideo.ifeng.com/', 'http://ips.ifeng.com/wideo.ifeng.com/'
    )
    _, ext, size = url_info(url)

    print_info(site_info, title, ext, size)
    if not info_only:
        download_urls([url], title, ext, size, **kwargs)


def ifeng_download(url, info_only=False, **kwargs):
    # old pattern /uuid.shtml
    # now it could be #uuid
    _id = match1(
        url,
        r'([0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12})'
    )
    if _id:
        return ifeng_download_by_id(_id, None, info_only=info_only, **kwargs)

    html = get_content(url)
    uuid_pattern = (
        r'"([0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12})"'
    )
    _id = match1(
        html,
        r'var vid="([0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-'
        '[0-9a-f]{12})"'
    )
    if _id is None:
        video_pattern = r'"vid"\s*:\s*' + uuid_pattern
        _id = match1(html, video_pattern)
    assert _id, "Can't find video info"
    return ifeng_download_by_id(_id, None, info_only=info_only, **kwargs)


download = ifeng_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import json

from lulu.common import (
    match1,
    player,
    url_size,
    print_info,
    get_content,
    download_urls,
    general_m3u8_extractor,
    playlist_not_supported,
)
from lulu.util.parser import get_parser


__all__ = ['longzhu_download']
site_info = '龙珠直播 longzhu.com'


def longzhu_download(url, info_only=False, **kwargs):
    web_domain = url.split('/')[2]
    if (web_domain == 'star.longzhu.com') or (web_domain == 'y.longzhu.com'):
        domain = url.split('/')[3].split('?')[0]
        m_url = 'http://m.longzhu.com/{}'.format(domain)
        m_html = get_content(m_url)
        room_id_patt = r'var\s*roomId\s*=\s*(\d+);'
        room_id = match1(m_html, room_id_patt)
        json_url = (
            'http://liveapi.plu.cn/liveapp/roomstatus?roomId={}'.format(
                room_id
            )
        )
        content = get_content(json_url)
        data = json.loads(content)
        streamUri = data['streamUri']
        if len(streamUri) <= 4:
            raise ValueError('The live stream is not online!')
        title = data['title']
        streamer = data['userName']
        title = '{}：{}'.format(streamer, title)

        steam_api_url = (
            'http://livestream.plu.cn/live/getlivePlayurl?roomId={}'.format(
                room_id
            )
        )
        content = get_content(steam_api_url)
        data = json.loads(content)
        isonline = data.get('isTransfer')
        if isonline == '0':
            raise ValueError('The live stream is not online!')

        real_url = data['playLines'][0]['urls'][0]['securityUrl']

        print_info(site_info, title, 'flv', float('inf'))

        if not info_only:
            download_urls([real_url], title, 'flv', None, **kwargs)

    elif web_domain == 'replay.longzhu.com':
        videoid = match1(url, r'(\d+)$')
        json_url = (
            'http://liveapi.longzhu.com/livereplay/getreplayfordisplay?'
            'videoId={}'.format(videoid)
        )
        content = get_content(json_url)
        data = json.loads(content)

        username = data['userName']
        title = data['title']
        title = '{}：{}'.format(username, title)
        real_url = data['videoUrl']

        print_info(site_info, title, 'm3u8', 0)
        if player:
            download_urls([real_url], title, 'm3u8', 0, **kwargs)
        else:
            urls = general_m3u8_extractor(real_url)
            if not info_only:
                download_urls(urls, title, 'ts', 0, **kwargs)

    elif web_domain == 'v.longzhu.com':
        page = get_content(url)
        parser = get_parser(page)
        title = parser.title.text
        media_id = match1(url, r'(\d+)$')
        # http://r.plures.net/ov/video/mobile/channel/video-1525da26174.js
        json_url = (
            'http://api.v.plu.cn/CloudMedia/GetInfoForPlayer?'
            'mediaId={}'.format(media_id)
        )
        content = get_content(json_url)
        data = json.loads(content)
        video = list(filter(lambda x: x['Ext'] == 'mp4', data['urls']))[0]
        video_url = video['SecurityUrl']
        ext = video['Ext']
        size = url_size(video_url)

        print_info(site_info, title, ext, size)
        if not info_only:
            download_urls([video_url], title, ext, size, **kwargs)

    else:
        raise ValueError('Wrong url or unsupported link ... {}'.format(url))


download = longzhu_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import os.path
import subprocess


def get_usable_rtmpdump(cmd):
    try:
        p = subprocess.Popen(
            [cmd], stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        out, err = p.communicate()
        return cmd
    except Exception:
        return None


RTMPDUMP = get_usable_rtmpdump('rtmpdump')


def has_rtmpdump_installed():
    return RTMPDUMP is not None


# params ={"-y":"playlist","-q":None,}
# if Only Key ,Value should be None
# -r -o should not be included in params


def download_rtmpdump_stream(url, title, ext, params={}, output_dir='.'):
    filename = '%s.%s' % (title, ext)
    filepath = os.path.join(output_dir, filename)

    cmdline = [RTMPDUMP, '-r']
    cmdline.append(url)
    cmdline.append('-o')
    cmdline.append(filepath)

    for key in params.keys():
        cmdline.append(key)
        if params[key] is not None:
            cmdline.append(params[key])

    # cmdline.append('-y')
    # cmdline.append(playpath)
    print("Call rtmpdump:\n"+" ".join(cmdline)+"\n")
    subprocess.call(cmdline)
    return


def play_rtmpdump_stream(player, url, params={}):
    # construct left side of pipe
    cmdline = [RTMPDUMP, '-r']
    cmdline.append(url)

    # append other params if exist
    for key in params.keys():
        cmdline.append(key)
        if params[key] is not None:
            cmdline.append(params[key])

    cmdline.append('-o')
    cmdline.append('-')

    # pipe start
    cmdline.append('|')
    cmdline.append(player)
    cmdline.append('-')

    # logging
    print('Call rtmpdump:\n{}\n'.format(' '.join(cmdline)))

    # call RTMPDump!
    subprocess.call(cmdline)

    # os.system("rtmpdump -r '%s' -y '%s' -o - | %s -" % (url, playpath, player))  # noqa
    return



#!/usr/bin/env python

import re
import json

from lulu.common import (
    match1,
    url_info,
    urls_size,
    get_content,
)
from lulu.extractor import VideoExtractor


class Imgur(VideoExtractor):
    name = 'Imgur'

    stream_types = [
        {'id': 'original'},
        {'id': 'thumbnail'},
    ]

    def prepare(self, **kwargs):
        if re.search(r'imgur\.com/a/', self.url):
            # album
            content = get_content(self.url)
            album = match1(content, r'album\s*:\s*({.*}),') or \
                match1(content, r'image\s*:\s*({.*}),')
            album = json.loads(album)
            # count = album['album_images']['count']
            images = album['album_images']['images']
            ext = images[0]['ext']
            self.streams = {
                'original': {
                    'src': [
                        'http://i.imgur.com/{}{}'.format(
                            i['hash'], ext
                        ) for i in images
                    ],
                    'size': sum([i['size'] for i in images]),
                    'container': ext[1:]
                },
                'thumbnail': {
                    'src': [
                        'http://i.imgur.com/{}s{}'.format(
                            i['hash'], '.jpg'
                        ) for i in images
                    ],
                    'container': 'jpg'
                }
            }
            self.title = album['title']

        elif re.search(r'i\.imgur\.com/', self.url):
            # direct image
            _, container, size = url_info(self.url)
            self.streams = {
                'original': {
                    'src': [self.url],
                    'size': size,
                    'container': container
                }
            }
            self.title = match1(self.url, r'i\.imgur\.com/([^./]*)')

        else:
            # gallery image
            content = get_content(self.url)
            image = json.loads(match1(content, r'image\s*:\s*({.*}),'))
            ext = image['ext']
            self.streams = {
                'original': {
                    'src': [
                        'http://i.imgur.com/{}{}'.format(image['hash'], ext)
                    ],
                    'size': image['size'],
                    'container': ext[1:]
                },
                'thumbnail': {
                    'src': [
                        'http://i.imgur.com/{}s{}'.format(
                            image['hash'], '.jpg'
                        )
                    ],
                    'container': 'jpg'
                }
            }
            self.title = image['title'] or image['hash']

    def extract(self, **kwargs):
        if 'stream_id' in kwargs and kwargs['stream_id']:
            i = kwargs['stream_id']
            if 'size' not in self.streams[i]:
                self.streams[i]['size'] = urls_size(self.streams[i]['src'])


site = Imgur()
download = site.download_by_url
download_playlist = site.download_by_url



#!/usr/bin/env python

import re
import json
from time import time
from urllib.parse import quote

from lulu.util import log
from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)


__all__ = ['pixnet_download']
site_info = '痞客邦 pixnet.net'


def pixnet_download(url, info_only=False, **kwargs):
    if not re.match(r'http://(\w)+.pixnet.net/album/video/(\d)+', url):
        log.wtf('[Failed] Unsupported URL pattern.')
        return
    # http://eric6513.pixnet.net/album/video/206644535
    html = get_content(url)
    title = ''.join(match1(
        html, r'<meta property="og:description\" content="([^"]*)"'
    ).split('-')[1:]).strip()

    time_now = int(time())

    m = re.match(r'http://(\w+).pixnet.net/album/video/(\d+)', url)

    username = m.group(1)
    # eric6513
    _id = m.group(2)
    # 206644535

    data_dict = {
        'username': username, 'autoplay': 1, 'id': _id, 'loop': 0,
        'profile': 9, 'time': time_now,
    }
    # have to be like this
    data_dict_str = quote(str(data_dict).replace("'", '"'), safe='"')
    url2 = 'http://api.pixnet.tv/content?type=json&customData={}'.format(
        data_dict_str
    )
    # &sig=edb07258e6a9ff40e375e11d30607983  can be blank for now
    # if required, can be obtained from url like
    # http://s.ext.pixnet.tv/user/eric6513/html5/autoplay/206644507.js
    # http://api.pixnet.tv/content?type=json&customData={%22username%22:%22eric6513%22,%22id%22:%22206644535%22,%22time%22:1441823350,%22autoplay%22:0,%22loop%22:0,%22profile%22:7}

    video_json = get_content(url2)
    content = json.loads(video_json)
    url_main = content['element']['video_url']
    url_backup = content['element']['backup_video_uri']

    try:
        # In some rare cases the main URL is IPv6 only...
        # Something like #611
        url_info(url_main)
        url = url_main
    except Exception:
        url = url_backup

    _type, ext, size = url_info(url)
    print_info(site_info, title, _type, size)
    if not info_only:
        download_urls([url], title, ext, size, **kwargs)


download = pixnet_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import re
import time
from urllib import (
    parse,
    request,
)

from lulu.common import (
    url_info,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)
from lulu.util import log
from lulu.config import FAKE_HEADERS


__all__ = ['vidto_download']
site_info = 'Vidto vidto.me'


def vidto_download(url, info_only=False, **kwargs):
    html = get_content(url)
    params = {}
    r = re.findall(
        r'type="(?:hidden|submit)?"(?:.*?)name="(.+?)"\s* value="?(.+?)">',
        html
    )
    for name, value in r:
        params[name] = value
    data = parse.urlencode(params).encode('utf-8')
    req = request.Request(url, headers=FAKE_HEADERS)
    print('Please wait for 6 seconds...')
    time.sleep(6)
    print('Starting')
    new_html = request.urlopen(req, data).read().decode('utf-8', 'replace')
    new_stff = re.search(r'lnk_download" href="(.*?)">', new_html)
    if new_stff:
        url = new_stff.group(1)
        title = params['fname']
        _type = ''
        ext = ''
        a, b, size = url_info(url)
        print_info(site_info, title, _type, size)
        if not info_only:
            download_urls([url], title, ext, size, **kwargs)
    else:
        log.wtf("Cann't find link, please review")


download = vidto_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import re
import json
from time import time, sleep

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_url_ffmpeg,
    playlist_not_supported,
)
from lulu.util import log
from lulu.config import FAKE_HEADERS_MOBILE


__all__ = ['showroom_download']
site_info = 'SHOWROOM showroom-live.com'


def showroom_get_roomid_by_room_url_key(room_url_key):
    """str->str
    """
    webpage_url = 'https://www.showroom-live.com/{}'.format(room_url_key)
    html = get_content(webpage_url, headers=FAKE_HEADERS_MOBILE)
    roomid = match1(html, r'room\?room_id\=(\d+)')
    assert roomid
    return roomid


def showroom_download_by_room_id(room_id, info_only=False, **kwargs):
    '''Source: Android mobile
    '''
    while True:
        timestamp = str(int(time() * 1000))
        api_endpoint = (
            'https://www.showroom-live.com/api/live/streaming_url?room_id='
            '{room_id}&_={timestamp}'.format(
                room_id=room_id, timestamp=timestamp
            )
        )
        html = get_content(api_endpoint)
        html = json.loads(html)
        if len(html) >= 1:
            break
        log.w('The live show is currently offline.')
        sleep(1)

    # This is mainly for testing the M3U FFmpeg parser so I would ignore
    # any non-m3u ones
    stream_url = [
        i['url'] for i in html['streaming_url_list']
        if i['is_default'] and i['type'] == 'hls'
    ][0]
    assert stream_url
    # title
    title = ''
    profile_api = (
        'https://www.showroom-live.com/api/room/profile?room_id='
        '{room_id}'.format(room_id=room_id)
    )
    html = json.loads(get_content(profile_api))
    try:
        title = html['main_name']
    except KeyError:
        title = 'Showroom_{room_id}'.format(room_id=room_id)

    type_, ext, size = url_info(stream_url)
    print_info(site_info, title, type_, size)
    if not info_only:
        download_url_ffmpeg(
            url=stream_url, title=title, ext='mp4', **kwargs
        )


def showroom_download(url, info_only=False, **kwargs):
    if re.match(r'(\w+)://www.showroom-live.com/([-\w]+)', url):
        room_url_key = match1(url, r'\w+://www.showroom-live.com/([-\w]+)')
        room_id = showroom_get_roomid_by_room_url_key(room_url_key)
        showroom_download_by_room_id(room_id, info_only, **kwargs)


download = showroom_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import re
import json

from lulu.common import (
    player,
    print_info,
    get_content,
    download_urls,
    launch_player,
    playlist_not_supported,
)


__all__ = ['yizhibo_download']
site_info = 'yizhibo.com'


def yizhibo_download(
    url, output_dir='.', merge=True, info_only=False, **kwargs
):
    video_id = url[url.rfind('/')+1:].split('.')[0]
    json_request_url = (
        'http://www.yizhibo.com/live/h5api/get_basic_live_info?'
        'scid={}'.format(video_id)
    )
    content = get_content(json_request_url)
    error = json.loads(content)['result']
    if error != 1:
        raise ValueError('Error : {}'.format(error))

    data = json.loads(content)
    title = data.get('data')['live_title']
    if (title == ''):
        title = data.get('data')['nickname']
    m3u8_url = data.get('data')['play_url']
    m3u8 = get_content(m3u8_url)
    base_url = '{}/'.format(
        '/'.join(data.get('data')['play_url'].split('/')[:7])
    )
    part_url = re.findall(r'([0-9]+\.ts)', m3u8)
    real_url = []
    for i in part_url:
        url = base_url + i
        real_url.append(url)
    print_info(site_info, title, 'ts', float('inf'))
    if not info_only:
        if player:
            launch_player(player, [m3u8_url])
        download_urls(
            real_url, title, 'ts', float('inf'), output_dir, merge=merge
        )


download = yizhibo_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

from hashlib import md5
from urllib.parse import urlparse

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)
from lulu.config import FAKE_HEADERS


__all__ = ['fc2video_download']
site_info = 'FC2 Video video.fc2.com'


def makeMimi(upid):
    """From http://cdn37.atwikiimg.com/sitescript/pub/dksitescript/FC2.site.js
    Also com.hps.util.fc2.FC2EncrptUtil.makeMimiLocal
    L110
    """
    strSeed = 'gGddgPfeaf_gzyr'
    prehash = '{}_{}'.format(upid, strSeed)
    return md5(prehash.encode('utf-8')).hexdigest()


def fc2video_download_by_upid(
    upid, output_dir='.', merge=True, info_only=False, **kwargs
):
    fake_headers = FAKE_HEADERS.copy()
    fake_headers.update({
        'DNT': '1',
        'Accept-Encoding': 'gzip, deflate, sdch',
        'Accept-Language': 'en-CA,en;q=0.8,en-US;q=0.6,zh-CN;q=0.4,zh;q=0.2',
        'X-Requested-With': 'ShockwaveFlash/19.0.0.245',
        'Connection': 'keep-alive',
    })
    api_base = (
        'https://video.fc2.com/ginfo.php?upid={upid}&mimi='
        '{mimi}'.format(upid=upid, mimi=makeMimi(upid))
    )
    html = get_content(api_base, headers=fake_headers)
    video_url = match1(html, r'filepath=(.+)&sec')
    video_url = video_url.replace('&mid', '?mid')

    title = match1(html, r'&title=([^&]+)')

    _type, ext, size = url_info(video_url, headers=fake_headers)
    print_info(site_info, title, _type, size)
    if not info_only:
        download_urls(
            [video_url], title, ext, size, output_dir, merge=merge,
            headers=fake_headers, **kwargs
        )


def fc2video_download(
    url, output_dir='.', merge=True, info_only=False, **kwargs
):
    """wrapper"""
    # 'http://video.fc2.com/en/content/20151021bTVKnbEw'
    # 'http://xiaojiadianvideo.asia/content/20151021bTVKnbEw'
    # 'http://video.fc2.com/ja/content/20151021bTVKnbEw'
    # 'http://video.fc2.com/tw/content/20151021bTVKnbEw'
    hostname = urlparse(url).hostname
    if not ('fc2.com' in hostname or 'xiaojiadianvideo.asia' in hostname):
        return False
    upid = match1(url, r'.+/content/(\w+)')

    fc2video_download_by_upid(upid, output_dir, merge, info_only)


download = fc2video_download
download_playlist = playlist_not_supported(site_info)



# coding=utf-8

from copy import copy
from xml.etree import cElementTree as ET

from lulu.common import (
    url_info,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)


__all__ = ['ckplayer_download']
site_info = 'CKPlayer General'


def ckplayer_get_info_by_xml(ckinfo):
    """str->dict
    Information for CKPlayer API content.
    """
    e = ET.XML(ckinfo)
    video_dict = {
        'title': '',
        'links': [],
        'size': 0,
        'flashvars': '',
    }
    dictified = dictify(e)['ckplayer']
    if 'info' in dictified:
        if '_text' in dictified['info'][0]['title'][0]:  # title
            video_dict['title'] = dictified['info'][0]['title'][0][
                '_text'
            ].strip()

    if '_text' in dictified['video'][0]['size'][0]:  # size exists for 1 piece
        video_dict['size'] = sum(
            [int(i['size'][0]['_text']) for i in dictified['video']]
        )

    if '_text' in dictified['video'][0]['file'][0]:  # link exist
        video_dict['links'] = [
            i['file'][0]['_text'].strip() for i in dictified['video']
        ]

    if '_text' in dictified['flashvars'][0]:
        video_dict['flashvars'] = dictified['flashvars'][0]['_text'].strip()

    return video_dict


# helper
# https://stackoverflow.com/questions/2148119/how-to-convert-an-xml-string-to-a-dictionary-in-python  # noqa
def dictify(r, root=True):
    if root:
        return {r.tag: dictify(r, False)}
    d = copy(r.attrib)
    if r.text:
        d['_text'] = r.text
    for x in r.findall('./*'):
        if x.tag not in d:
            d[x.tag] = []
        d[x.tag].append(dictify(x, False))
    return d


def ckplayer_download_by_xml(
    ckinfo, output_dir='.', merge=False, info_only=False, **kwargs
):
    # Info XML
    video_info = ckplayer_get_info_by_xml(ckinfo)

    try:
        title = kwargs['title']
    except Exception:
        title = ''
    type_ = ''
    size = 0

    if len(video_info['links']) > 0:  # has link
        # use 1st to determine type, ext
        type_, _ext, size = url_info(video_info['links'][0])

    if 'size' in video_info:
        size = int(video_info['size'])
    else:
        for i in video_info['links'][1:]:  # save 1st one
            size += url_info(i)[2]

    print_info(site_info, title, type_, size)
    if not info_only:
        download_urls(
            video_info['links'], title, _ext, size, output_dir=output_dir,
            merge=merge
        )


def ckplayer_download(
    url, output_dir='.', merge=False, info_only=False, is_xml=True, **kwargs
):
    if is_xml:  # URL is XML URL
        try:
            title = kwargs['title']
        except Exception:
            title = ''
        try:
            headers = kwargs['headers']  # headers provided
            ckinfo = get_content(url, headers=headers)
        except NameError:
            ckinfo = get_content(url)

        ckplayer_download_by_xml(
            ckinfo, output_dir, merge, info_only, title=title
        )


download = ckplayer_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

from lulu.common import (
    match1,
    print_info,
    get_content,
    download_url_ffmpeg,
    playlist_not_supported,
)
from lulu.util.parser import get_parser


__all__ = ['huomaotv_download']
site_info = '火猫直播 huomao.com'


def huomaotv_download(
    url, output_dir='.', merge=True, info_only=False, **kwargs
):
    room_id_pattern = r'huomao.com/(\d+)'
    room_id = match1(url, room_id_pattern)
    html = get_content(
        'http://m.huomao.com/mobile/mob_live/{}'.format(room_id)
    )
    parser = get_parser(html)
    m3u8_url = parser.source['src']
    title = parser.title.text

    print_info(site_info, title, 'm3u8', float('inf'))

    if not info_only:
        download_url_ffmpeg(
            m3u8_url, title, 'mp4', None, output_dir=output_dir, merge=merge
        )


download = huomaotv_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import re
import json
import base64
import random
import binascii
from urllib import parse

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
)


__all__ = ['ixigua_download', 'ixigua_download_playlist']
site_info = '西瓜视频 ixigua.com'


def get_r():
    return str(random.random())[2:]


def right_shift(val, n):
    return val >> n if val >= 0 else (val + 0x100000000) >> n


def get_s(text):
    """get video info
    """
    _id = match1(text, r"videoId: '(.*?)'")
    p = get_r()
    url = 'http://i.snssdk.com/video/urls/v/1/toutiao/mp4/{}'.format(_id)
    n = '{}?r={}'.format(parse.urlparse(url).path, p)
    c = binascii.crc32(n.encode('utf-8'))
    s = right_shift(c, 0)
    title = ''.join(re.findall(r"title: '(.*?)',", text))
    v_url = '{}?r={}&s={}'.format(url, p, s)
    return v_url, title


def get_moment(url, user_id, base_url, video_list):
    """Recursively obtaining a video list
    """
    video_list_data = json.loads(get_content(url))
    if not video_list_data['next']['max_behot_time']:
        return video_list
    [video_list.append(i['display_url']) for i in video_list_data['data']]
    max_behot_time = video_list_data['next']['max_behot_time']
    _param = {
        'user_id': user_id,
        'base_url': base_url,
        'video_list': video_list,
        'url': base_url.format(user_id=user_id, max_behot_time=max_behot_time),
    }
    return get_moment(**_param)


def ixigua_download(url, output_dir='.', info_only=False, **kwargs):
    """Download a single video
    Sample URL: https://www.ixigua.com/a6487187567887254029/#mid=59051127876
    """
    if 'toutiao.com/group' in url:
        url = 'https://www.ixigua.com/a{}'.format(
            match1(url, r'.*?toutiao.com/group/(\d+)')
        )
    try:
        video_info_url, title = get_s(get_content(url))
        video_info = json.loads(get_content(video_info_url))
    except Exception:
        raise NotImplementedError(url)
    try:
        video_url = base64.b64decode(
            video_info['data']['video_list']['video_1']['main_url']
        ).decode()
    except Exception:
        raise NotImplementedError(url)
    filetype, ext, size = url_info(video_url)
    print_info(site_info, title, filetype, size)
    if not info_only:
        download_urls([video_url], title, ext, size, output_dir=output_dir)


def ixigua_download_playlist(url, output_dir='.', info_only=False, **kwargs):
    """Download all video from the user's video list
    Sample URL: https://www.ixigua.com/c/user/71141690831/
    """
    if 'user' not in url:
        raise NotImplementedError(url)
    user_id = url.split('/')[-2]
    max_behot_time = 0
    if not user_id:
        raise NotImplementedError(url)
    base_url = (
        'https://www.ixigua.com/c/user/article/?user_id={user_id}&'
        'max_behot_time={max_behot_time}&max_repin_time=0&count=20&page_type=0'
    )
    _param = {
        'user_id': user_id,
        'base_url': base_url,
        'video_list': [],
        'url': base_url.format(user_id=user_id, max_behot_time=max_behot_time),
    }
    for i in get_moment(**_param):
        ixigua_download(i, output_dir, info_only, **kwargs)


download = ixigua_download
download_playlist = ixigua_download_playlist



#!/usr/bin/env python

from lulu.common import (
    url_info,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)
from lulu.util.parser import get_parser


__all__ = ['joy_download']
site_info = '激动网 joy.cn'


def joy_download(url, info_only=False, **kwargs):
    page = get_content(url)
    parser = get_parser(page)
    url = parser.source['src']
    title = parser.h1.text.strip()
    _, ext, size = url_info(url)
    print_info(site_info, title, ext, size)
    if not info_only:
        download_urls([url], title, ext, size, **kwargs)


download = joy_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import re
import json
import time
import hashlib

from lulu.common import (
    match1,
    get_content,
)
from lulu.util import log
from lulu.util.parser import get_parser
from lulu.extractor import VideoExtractor


'''
Changelog:
-> http://www.iqiyi.com/common/flashplayer/20150916/MainPlayer_5_2_28_c3_3_7_4.swf
   use @fffonion 's method in #617.
   Add trace AVM(asasm) code in Iqiyi's encode function where the salt is put into the encode array and reassemble by RABCDasm(or WinRABCDasm),then use Fiddler to response modified file to replace the src file with its AutoResponder function ,set browser Fiddler proxy and play with !debug version! Flash Player ,finially get result in flashlog.txt(its location can be easily found in search engine).
   Code Like (without letters after #comment:),it just do the job : trace("{IQIYI_SALT}:"+salt_array.join(""))
   ```(Postion After getTimer)
     findpropstrict      QName(PackageNamespace(""), "trace")
     pushstring          "{IQIYI_SALT}:" #comment for you to locate the salt
     getscopeobject      1
     getslot             17 #comment: 17 is the salt slots number defined in code
     pushstring          ""
     callproperty        QName(Namespace("http://adobe.com/AS3/2006/builtin"), "join"), 1
     add
     callpropvoid        QName(PackageNamespace(""), "trace"), 1
   ```

-> http://www.iqiyi.com/common/flashplayer/20150820/MainPlayer_5_2_27_2_c3_3_7_3.swf
    some small changes in Zombie.bite function

'''  # noqa

'''
com.qiyi.player.core.model.def.DefinitonEnum
bid meaning for quality
0 none
1 standard
2 high
3 super
4 suprt-high
5 fullhd
10 4k
96 topspeed
'''


class Iqiyi(VideoExtractor):
    name = '爱奇艺 iqiyi.com'

    stream_types = [
        {'id': '4k', 'container': 'm3u8', 'video_profile': '4k'},
        {'id': 'BD', 'container': 'm3u8', 'video_profile': '1080p'},
        {'id': 'TD', 'container': 'm3u8', 'video_profile': '720p'},
        {'id': 'TD_H265', 'container': 'm3u8', 'video_profile': '720p H265'},
        {'id': 'HD', 'container': 'm3u8', 'video_profile': '540p'},
        {'id': 'HD_H265', 'container': 'm3u8', 'video_profile': '540p H265'},
        {'id': 'SD', 'container': 'm3u8', 'video_profile': '360p'},
        {'id': 'LD', 'container': 'm3u8', 'video_profile': '210p'},
    ]
    ids = ['4k', 'BD', 'TD', 'HD', 'SD', 'LD']
    vd_2_id = {
        10: '4k', 19: '4k', 5: 'BD', 18: 'BD', 21: 'HD_H265', 2: 'HD',
        4: 'TD', 17: 'TD_H265', 96: 'LD', 1: 'SD',
    }
    # http://www.iqiyi.com/v_19rr7p0jss.html
    # 14: 'TD' 中的 m3u8 地址中的文件有问题
    id_2_profile = {
        '4k': '4k', 'BD': '1080p', 'TD': '720p', 'HD': '540p', 'SD': '360p',
        'LD': '210p', 'HD_H265': '540p H265', 'TD_H265': '720p H265',
    }

    def get_raw_data(self, tvid, vid):
        t = int(time.time() * 1000)
        src = '76f90cbd92f94a2e925d83e8ccd22cb7'
        key = 'd5fb4bd9d50c4be6948c97edd7254b0e'
        sc = hashlib.new(
            'md5', bytes(str(t) + key + vid, 'utf-8')
        ).hexdigest()
        vmsreq = (
            'http://cache.m.iqiyi.com/jp/tmts/{}/{}/?t={}&sc={}&src={}'.format(
                tvid, vid, t, sc, src
            )
        )
        return json.loads(get_content(vmsreq)[len('var tvInfoJs='):])

    def download_playlist_by_url(self, url, **kwargs):
        self.url = url

        video_page = get_content(url)
        videos = set(re.findall(
            r'<a href="(http://www\.iqiyi\.com/v_[^"]+)"', video_page
        ))

        for video in videos:
            self.__class__().download_by_url(video, **kwargs)

    def prepare(self, **kwargs):
        assert self.url or self.vid

        if self.url and not self.vid:
            html = get_content(self.url)
            tvid = match1(self.url, r'#curid=(.+)_') or match1(
                self.url, r'tvid=([^&]+)'
            ) or match1(html, r'data-player-tvid="([^"]+)"') or match1(
                html, r'tv(?:i|I)d=(.+?)\&'
            ) or match1(html, r'param\[\'tvid\'\]\s*=\s*"(.+?)"')
            videoid = match1(self.url, r'#curid=.+_(.*)$') or match1(
                self.url, r'vid=([^&]+)'
            ) or match1(html, r'data-player-videoid="([^"]+)"') or match1(
                html, r'vid=(.+?)\&'
            ) or match1(html, r'param\[\'vid\'\]\s*=\s*"(.+?)"')
            self.vid = (tvid, videoid)
            info_u = 'http://mixer.video.iqiyi.com/jp/mixin/videos/{}'.format(
                tvid
            )
            mixin = get_content(info_u)
            mixin_json = json.loads(mixin[len('var tvInfoJs='):])
            real_u = mixin_json['url']
            real_html = get_content(real_u)
            parser = get_parser(real_html)
            self.title = parser.find('meta', property='og:title')['content']
        tvid, videoid = self.vid
        info = self.get_raw_data(tvid, videoid)
        assert info['code'] == 'A00000', "Can't play this video"
        for stream in info['data']['vidl']:
            if 'm3utx' not in stream:
                continue
            try:
                stream_id = self.vd_2_id[stream['vd']]
                stream_profile = self.id_2_profile[stream_id]
                self.streams[stream_id] = {
                    'video_profile': stream_profile,
                    'container': 'm3u8',
                    'src': [stream['m3utx']],
                    'size': 0,
                    'm3u8_url': stream['m3utx'],
                }
            except Exception as e:
                pass
                # log.i('vd: {} is not handled'.format(stream['vd']))
                # log.i('info is {}'.format(stream))


site = Iqiyi()
download = site.download_by_url
iqiyi_download_by_vid = site.download_by_vid
download_playlist = site.download_playlist_by_url



#!/usr/bin/env python

import json

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)
from lulu.config import FAKE_HEADERS


headers = FAKE_HEADERS.copy()
headers.update({
    'DNT': '1',
    'Accept-Encoding': 'gzip, deflate, sdch, br',
    'Accept-Language': 'en-CA,en;q=0.8,en-US;q=0.6,zh-CN;q=0.4,zh;q=0.2',
    'Upgrade-Insecure-Requests': '1',
    'Connection': 'keep-alive',
    'Save-Data': 'on',
    'Cookie': 'has_js=1;show_adult=1',
})


__all__ = ['iwara_download']
site_info = 'Iwara iwara.tv'


def iwara_download(url, info_only=False, **kwargs):
    video_hash = match1(url, r'http://\w+.iwara.tv/videos/(\w+)')
    video_url = match1(url, r'(http://\w+.iwara.tv)/videos/\w+')
    html = get_content(url, headers=headers)
    title = match1(html, r'<title>(.*)</title>')
    api_url = '{}/api/video/{}'.format(video_url, video_hash)
    content = get_content(api_url, headers=headers)
    data = json.loads(content)
    _type, ext, size = url_info(data[0]['uri'], headers=headers)
    down_urls = data[0]['uri']
    print_info(down_urls, title, _type, size)

    if not info_only:
        download_urls([down_urls], title, ext, size, headers=headers, **kwargs)


download = iwara_download
download_playlist = playlist_not_supported(site_info)



# coding=utf-8

from html import unescape

from lulu.util.fs import legitimize


def get_filename(htmlstring):
    return legitimize(unescape(htmlstring))


def parameterize(string):
    return "'{}'".format(string.replace("'", r"'\''"))



#!/usr/bin/env python

import re
import json
from urllib import parse
from html import unescape
from xml.dom.minidom import parseString

from lulu.common import (
    match1,
    url_size,
    urls_size,
    get_content,
    mime_to_container,
    parse_query_param,
    download_url_ffmpeg,
)
from lulu.util import log
from lulu.config import YOUTUBE_CODECS
from lulu.extractor import VideoExtractor


class YouTube(VideoExtractor):
    name = 'YouTube'

    # YouTube media encoding options, in descending quality order.
    # http://en.wikipedia.org/wiki/YouTube#Quality_and_codecs.
    # Retrieved July 17, 2014.
    stream_types = YOUTUBE_CODECS

    def decipher(js, s):
        def tr_js(code):
            code = re.sub(r'function', r'def', code)
            code = re.sub(r'(\W)(as|if|in|is|or)\(', r'\1_\2(', code)
            code = re.sub(r'\$', '_dollar', code)
            code = re.sub(r'\{', r':\n\t', code)
            code = re.sub(r'\}', r'\n', code)
            code = re.sub(r'var\s+', r'', code)
            code = re.sub(r'(\w+).join\(""\)', r'"".join(\1)', code)
            code = re.sub(r'(\w+).length', r'len(\1)', code)
            code = re.sub(r'(\w+).slice\((\w+)\)', r'\1[\2:]', code)
            code = re.sub(
                r'(\w+).splice\((\w+),(\w+)\)', r'del \1[\2:\2+\3]', code
            )
            code = re.sub(r'(\w+).split\(""\)', r'list(\1)', code)
            return code

        js = js.replace('\n', ' ')
        f1 = match1(js, r'"signature",([$\w]+)\(\w+\.\w+\)')
        f1def = match1(
            js, r'function %s(\(\w+\)\{[^\{]+\})' % re.escape(f1)
        ) or match1(js, r'\W%s=function(\(\w+\)\{[^\{]+\})' % re.escape(f1))
        f1def = re.sub(r'([$\w]+\.)([$\w]+\(\w+,\d+\))', r'\2', f1def)
        f1def = 'function %s%s' % (f1, f1def)
        code = tr_js(f1def)
        f2s = set(re.findall(r'([$\w]+)\(\w+,\d+\)', f1def))
        for f2 in f2s:
            f2e = re.escape(f2)
            f2def = re.search(
                r'[^$\w]%s:function\((\w+,\w+)\)(\{[^\{\}]+\})' % f2e, js
            )
            if f2def:
                f2def = 'function {}({}){}'.format(
                    f2e, f2def.group(1), f2def.group(2)
                )
            else:
                f2def = re.search(
                    r'[^$\w]%s:function\((\w+)\)(\{[^\{\}]+\})' % f2e, js
                )
                f2def = 'function {}({},b){}'.format(
                    f2e, f2def.group(1), f2def.group(2)
                )
            f2 = re.sub(r'(\W)(as|if|in|is|or)\(', r'\1_\2(', f2)
            f2 = re.sub(r'\$', '_dollar', f2)
            code = code + 'global %s\n' % f2 + tr_js(f2def)

        f1 = re.sub(r'(as|if|in|is|or)', r'_\1', f1)
        f1 = re.sub(r'\$', '_dollar', f1)
        code = code + 'sig=%s(s)' % f1
        exec(code, globals(), locals())
        return locals()['sig']

    def get_url_from_vid(vid):
        return 'https://youtu.be/{}'.format(vid)

    def get_vid_from_url(url):
        """Extracts video ID from URL.
        """
        return match1(url, r'youtu\.be/([^?/]+)') or \
            match1(url, r'youtube\.com/embed/([^/?]+)') or \
            match1(url, r'youtube\.com/v/([^/?]+)') or \
            match1(url, r'youtube\.com/watch/([^/?]+)') or \
            parse_query_param(url, 'v') or \
            parse_query_param(parse_query_param(url, 'u'), 'v')

    def get_playlist_id_from_url(url):
        """Extracts playlist ID from URL.
        """
        return parse_query_param(url, 'list') or \
            parse_query_param(url, 'p')

    def download_playlist_by_url(self, url, **kwargs):
        self.url = url

        playlist_id = self.__class__.get_playlist_id_from_url(self.url)
        if playlist_id is None:
            log.wtf('[Failed] Unsupported URL pattern.')

        video_page = get_content(
            'https://www.youtube.com/playlist?list={}'.format(playlist_id)
        )
        from html.parser import HTMLParser
        videos = sorted(
            [HTMLParser().unescape(video) for video in re.findall(
                r'<a href="(/watch\?[^"]+)"', video_page
            ) if parse_query_param(video, 'index')],
            key=lambda video: parse_query_param(video, 'index')
        )

        # Parse browse_ajax page for more videos to load
        load_more_href = match1(
            video_page, r'data-uix-load-more-href="([^"]+)"'
        )
        while load_more_href:
            browse_ajax = get_content(
                'https://www.youtube.com/{}'.format(load_more_href)
            )
            browse_data = json.loads(browse_ajax)
            load_more_widget_html = browse_data['load_more_widget_html']
            content_html = browse_data['content_html']
            vs = set(re.findall(r'href="(/watch\?[^"]+)"', content_html))
            videos += sorted(
                [HTMLParser().unescape(video) for video in list(vs)
                    if parse_query_param(video, 'index')]
            )
            load_more_href = match1(
                load_more_widget_html, r'data-uix-load-more-href="([^"]+)"'
            )

        self.title = re.search(
            r'<meta name="title" content="([^"]+)"', video_page
        ).group(1)
        self.p_playlist()
        for video in videos:
            vid = parse_query_param(video, 'v')
            index = parse_query_param(video, 'index')
            self.__class__().download_by_url(
                self.__class__.get_url_from_vid(vid), index=index, **kwargs
            )

    def prepare(self, **kwargs):
        assert self.url or self.vid

        if not self.vid and self.url:
            self.vid = self.__class__.get_vid_from_url(self.url)

            if self.vid is None:
                self.download_playlist_by_url(self.url, **kwargs)
                exit(0)

        video_info = parse.parse_qs(
            get_content(
                'https://www.youtube.com/get_video_info?video_id={}'.format(
                    self.vid
                )
            )
        )

        ytplayer_config = None
        if 'status' not in video_info:
            log.wtf('[Failed] Unknown status.')
        elif video_info['status'] == ['ok']:
            if 'use_cipher_signature' not in video_info \
                    or video_info['use_cipher_signature'] == ['False']:
                self.title = parse.unquote_plus(video_info['title'][0])

                # Parse video page (for DASH)
                video_page = get_content(
                    'https://www.youtube.com/watch?v={}'.format(self.vid)
                )
                try:
                    ytplayer_config = json.loads(re.search(
                        'ytplayer.config\s*=\s*([^\n]+?});', video_page
                    ).group(1))
                    self.html5player = 'https://www.youtube.com{}'.format(
                        ytplayer_config['assets']['js']
                    )
                    # Workaround: get_video_info returns bad s. Why?
                    stream_list = ytplayer_config['args'][
                        'url_encoded_fmt_stream_map'
                    ].split(',')
                except Exception:
                    stream_list = video_info[
                        'url_encoded_fmt_stream_map'
                    ][0].split(',')
                    self.html5player = None

            else:
                # Parse video page instead
                video_page = get_content(
                    'https://www.youtube.com/watch?v={}'.format(self.vid)
                )
                ytplayer_config = json.loads(re.search(
                    'ytplayer.config\s*=\s*([^\n]+?});', video_page
                ).group(1))

                self.title = ytplayer_config['args']['title']
                self.html5player = 'https://www.youtube.com{}'.format(
                    ytplayer_config['assets']['js']
                )
                stream_list = ytplayer_config['args'][
                    'url_encoded_fmt_stream_map'
                ].split(',')

        elif video_info['status'] == ['fail']:
            if video_info['errorcode'] == ['150']:
                video_page = get_content(
                    'https://www.youtube.com/watch?v={}'.format(self.vid)
                )
                try:
                    ytplayer_config = json.loads(re.search(
                        'ytplayer.config\s*=\s*([^\n]+});ytplayer', video_page
                    ).group(1))
                except Exception:
                    msg = re.search(
                        'class="message">([^<]+)<', video_page
                    ).group(1)
                    log.wtf('[Failed] "%s"' % msg.strip())

                if 'title' in ytplayer_config['args']:
                    # 150 Restricted from playback on certain sites
                    # Parse video page instead
                    self.title = ytplayer_config['args']['title']
                    self.html5player = 'https://www.youtube.com{}'.format(
                        ytplayer_config['assets']['js']
                    )
                    stream_list = ytplayer_config['args'][
                        'url_encoded_fmt_stream_map'
                    ].split(',')
                else:
                    log.wtf(
                        '[Error] The uploader has not made this video '
                        'available in your country.'
                    )

            elif video_info['errorcode'] == ['100']:
                log.wtf(
                    '[Failed] This video does not exist.',
                    exit_code=int(video_info['errorcode'][0])
                )

            else:
                log.wtf(
                    '[Failed] %s' % video_info['reason'][0],
                    exit_code=int(video_info['errorcode'][0])
                )

        else:
            log.wtf('[Failed] Invalid status.')

        # YouTube Live
        if ytplayer_config and (
            ytplayer_config['args'].get('livestream') == '1' or
            ytplayer_config['args'].get('live_playback') == '1'
        ):
            hlsvp = ytplayer_config['args']['hlsvp']

            if 'info_only' in kwargs and kwargs['info_only']:
                return
            else:
                download_url_ffmpeg(hlsvp, self.title, 'mp4')
                exit(0)

        for stream in stream_list:
            metadata = parse.parse_qs(stream)
            stream_itag = metadata['itag'][0]
            self.streams[stream_itag] = {
                'itag': metadata['itag'][0],
                'url': metadata['url'][0],
                'sig': metadata['sig'][0] if 'sig' in metadata else None,
                's': metadata['s'][0] if 's' in metadata else None,
                'quality': metadata['quality'][0],
                'type': metadata['type'][0],
                'mime': metadata['type'][0].split(';')[0],
                'container': mime_to_container(
                    metadata['type'][0].split(';')[0]
                ),
            }

        # Prepare caption tracks
        try:
            caption_tracks = json.loads(
                ytplayer_config['args']['player_response']
            )['captions']['playerCaptionsTracklistRenderer']['captionTracks']
            for ct in caption_tracks:
                ttsurl, lang = ct['baseUrl'], ct['languageCode']

                tts_xml = parseString(get_content(ttsurl))
                transcript = tts_xml.getElementsByTagName('transcript')[0]
                texts = transcript.getElementsByTagName('text')
                srt = ''
                seq = 0
                for text in texts:
                    if text.firstChild is None:
                        continue  # empty element
                    seq += 1
                    start = float(text.getAttribute('start'))
                    if text.getAttribute('dur'):
                        dur = float(text.getAttribute('dur'))
                    else:
                        dur = 1.0  # could be ill-formed XML
                    finish = start + dur
                    m, s = divmod(start, 60)
                    h, m = divmod(m, 60)
                    start = '{:0>2}:{:0>2}:{:06.3f}'.format(
                        int(h), int(m), s
                    ).replace('.', ',')
                    m, s = divmod(finish, 60)
                    h, m = divmod(m, 60)
                    finish = '{:0>2}:{:0>2}:{:06.3f}'.format(
                        int(h), int(m), s
                    ).replace('.', ',')
                    content = unescape(text.firstChild.nodeValue)

                    srt += '{}\n'.format(str(seq))
                    srt += '{} --> {}\n'.format(start, finish)
                    srt += '{}\n\n'.format(content)

                self.caption_tracks[lang] = srt
        except Exception:
            pass

        # Prepare DASH streams
        try:
            dashmpd = ytplayer_config['args']['dashmpd']
            dash_xml = parseString(get_content(dashmpd))
            for aset in dash_xml.getElementsByTagName('AdaptationSet'):
                mimeType = aset.getAttribute('mimeType')
                if mimeType == 'audio/mp4':
                    rep = aset.getElementsByTagName('Representation')[-1]
                    burls = rep.getElementsByTagName('BaseURL')
                    dash_mp4_a_url = burls[0].firstChild.nodeValue
                    dash_mp4_a_size = burls[0].getAttribute('yt:contentLength')
                    if not dash_mp4_a_size:
                        try:
                            dash_mp4_a_size = url_size(dash_mp4_a_url)
                        except Exception:
                            continue
                elif mimeType == 'audio/webm':
                    rep = aset.getElementsByTagName('Representation')[-1]
                    burls = rep.getElementsByTagName('BaseURL')
                    dash_webm_a_url = burls[0].firstChild.nodeValue
                    dash_webm_a_size = burls[0].getAttribute(
                        'yt:contentLength'
                    )
                    if not dash_webm_a_size:
                        try:
                            dash_webm_a_size = url_size(dash_webm_a_url)
                        except Exception:
                            continue
                elif mimeType == 'video/mp4':
                    for rep in aset.getElementsByTagName('Representation'):
                        w = int(rep.getAttribute('width'))
                        h = int(rep.getAttribute('height'))
                        itag = rep.getAttribute('id')
                        burls = rep.getElementsByTagName('BaseURL')
                        dash_url = burls[0].firstChild.nodeValue
                        dash_size = burls[0].getAttribute('yt:contentLength')
                        if not dash_size:
                            try:
                                dash_size = url_size(dash_url)
                            except Exception:
                                continue
                        self.dash_streams[itag] = {
                            'quality': '{}x{}'.format(w, h),
                            'itag': itag,
                            'type': mimeType,
                            'mime': mimeType,
                            'container': 'mp4',
                            'src': [dash_url, dash_mp4_a_url],
                            'size': int(dash_size) + int(dash_mp4_a_size)
                        }
                elif mimeType == 'video/webm':
                    for rep in aset.getElementsByTagName('Representation'):
                        w = int(rep.getAttribute('width'))
                        h = int(rep.getAttribute('height'))
                        itag = rep.getAttribute('id')
                        burls = rep.getElementsByTagName('BaseURL')
                        dash_url = burls[0].firstChild.nodeValue
                        dash_size = burls[0].getAttribute('yt:contentLength')
                        if not dash_size:
                            try:
                                dash_size = url_size(dash_url)
                            except Exception:
                                continue
                        self.dash_streams[itag] = {
                            'quality': '%sx%s' % (w, h),
                            'itag': itag,
                            'type': mimeType,
                            'mime': mimeType,
                            'container': 'webm',
                            'src': [dash_url, dash_webm_a_url],
                            'size': int(dash_size) + int(dash_webm_a_size)
                        }
        except Exception:
            # VEVO
            if not self.html5player:
                return
            self.js = get_content(self.html5player)
            if 'adaptive_fmts' in ytplayer_config['args']:
                streams = [
                    dict(
                        [
                            (
                                i.split('=')[0],
                                parse.unquote(i.split('=')[1])
                            ) for i in afmt.split('&')
                        ]
                    ) for afmt in ytplayer_config['args'][
                        'adaptive_fmts'
                    ].split(',')
                ]
                for stream in streams:  # get over speed limiting
                    stream['url'] += '&ratebypass=yes'
                for stream in streams:  # audio
                    if stream['type'].startswith('audio/mp4'):
                        dash_mp4_a_url = stream['url']
                        if 's' in stream:
                            sig = self.__class__.decipher(self.js, stream['s'])
                            dash_mp4_a_url += '&signature={}'.format(sig)
                        dash_mp4_a_size = stream['clen']
                    elif stream['type'].startswith('audio/webm'):
                        dash_webm_a_url = stream['url']
                        if 's' in stream:
                            sig = self.__class__.decipher(self.js, stream['s'])
                            dash_webm_a_url += '&signature={}'.format(sig)
                        dash_webm_a_size = stream['clen']
                for stream in streams:  # video
                    if 'size' in stream:
                        if stream['type'].startswith('video/mp4'):
                            mimeType = 'video/mp4'
                            dash_url = stream['url']
                            if 's' in stream:
                                sig = self.__class__.decipher(
                                    self.js, stream['s']
                                )
                                dash_url += '&signature={}'.format(sig)
                            dash_size = stream['clen']
                            itag = stream['itag']
                            self.dash_streams[itag] = {
                                'quality': stream['size'],
                                'itag': itag,
                                'type': mimeType,
                                'mime': mimeType,
                                'container': 'mp4',
                                'src': [dash_url, dash_mp4_a_url],
                                'size': int(dash_size) + int(dash_mp4_a_size)
                            }
                        elif stream['type'].startswith('video/webm'):
                            mimeType = 'video/webm'
                            dash_url = stream['url']
                            if 's' in stream:
                                sig = self.__class__.decipher(
                                    self.js, stream['s']
                                )
                                dash_url += '&signature={}'.format(sig)
                            dash_size = stream['clen']
                            itag = stream['itag']
                            self.dash_streams[itag] = {
                                'quality': stream['size'],
                                'itag': itag,
                                'type': mimeType,
                                'mime': mimeType,
                                'container': 'webm',
                                'src': [dash_url, dash_webm_a_url],
                                'size': int(dash_size) + int(dash_webm_a_size)
                            }

    def extract(self, **kwargs):
        if not self.streams_sorted:
            # No stream is available
            return

        if 'stream_id' in kwargs and kwargs['stream_id']:
            # Extract the stream
            stream_id = kwargs['stream_id']
            if stream_id not in self.streams \
                    and stream_id not in self.dash_streams:
                log.e('[Error] Invalid video format.')
                log.e(
                    'Run \'-i\' command with no specific video format to '
                    'view all available formats.'
                )
                exit(2)
        else:
            # Extract stream with the best quality
            stream_id = self.streams_sorted[0]['itag']

        if stream_id in self.streams:
            src = self.streams[stream_id]['url']
            if self.streams[stream_id]['sig'] is not None:
                sig = self.streams[stream_id]['sig']
                src += '&signature={}'.format(sig)
            elif self.streams[stream_id]['s'] is not None:
                if not hasattr(self, 'js'):
                    self.js = get_content(self.html5player)
                s = self.streams[stream_id]['s']
                sig = self.__class__.decipher(self.js, s)
                src += '&signature={}'.format(sig)

            self.streams[stream_id]['src'] = [src]
            self.streams[stream_id]['size'] = urls_size(
                self.streams[stream_id]['src']
            )


site = YouTube()
download = site.download_by_url
download_playlist = site.download_playlist_by_url



#!/usr/bin/env python

import re
import json

from lulu.common import (
    match1,
    url_info,
    url_size,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)
from lulu.util.parser import get_parser


__all__ = ['instagram_download']
site_info = 'Instagram.com'


def instagram_download(
    url, output_dir='.', merge=True, info_only=False, **kwargs
):
    url = match1(url, r'([^?]*)')
    html = get_content(url)

    vid = match1(url, r'instagram.com/p/([^/]+)')
    parser = get_parser(html)
    description = parser.find('meta', property='og:title')['content']
    title = '{} [{}]'.format(description, vid)
    stream = parser.find('meta', property='og:video')
    if stream:
        stream = stream['content']
        _, ext, size = url_info(stream)

        print_info(site_info, title, ext, size)
        if not info_only:
            download_urls([stream], title, ext, size, output_dir, merge=merge)
    else:
        data = re.search(r'window\._sharedData\s*=\s*(.*);</script>', html)
        info = json.loads(data.group(1))

        if 'edge_sidecar_to_children' in info['entry_data']['PostPage'][0][
            'graphql'
        ]['shortcode_media']:
            edges = info['entry_data']['PostPage'][0]['graphql'][
                'shortcode_media'
            ]['edge_sidecar_to_children']['edges']
            for edge in edges:
                title = edge['node']['shortcode']
                image_url = edge['node']['display_url']
                ext = image_url.split('.')[-1]
                size = url_size(image_url)
                print_info(site_info, title, ext, size)
                if not info_only:
                    download_urls(
                        urls=[image_url], title=title, ext=ext,
                        total_size=size, output_dir=output_dir
                    )
        else:
            title = info['entry_data']['PostPage'][0]['graphql'][
                'shortcode_media'
            ]['shortcode']
            image_url = info['entry_data']['PostPage'][0]['graphql'][
                'shortcode_media'
            ]['display_url']
            ext = image_url.split('.')[-1]
            size = url_size(image_url)
            print_info(site_info, title, ext, size)
            if not info_only:
                download_urls(
                    urls=[image_url], title=title, ext=ext, total_size=size,
                    output_dir=output_dir
                )


download = instagram_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import time
import json
import socket
import hashlib
import urllib.parse
import urllib.request
from xml.dom.minidom import parseString

from lulu.common import (
    match1,
    get_content,
    url_locations,
)
from lulu.util import log
from lulu.config import FAKE_HEADERS
from lulu.util.parser import get_parser
from lulu.extractor import VideoExtractor
from lulu.extractors.qq import qq_download_by_vid
from lulu.extractors.sina import sina_download_by_vid
from lulu.extractors.youku import youku_download_by_vid


__all__ = ['bilibili_download']


class Bilibili(VideoExtractor):
    name = '哔哩哔哩 bilibili.com'
    live_api = 'https://live.bilibili.com/api/playurl?cid={}&otype=json'
    api_url = 'https://interface.bilibili.com/playurl?'
    bangumi_api_url = 'https://bangumi.bilibili.com/player/web_api/playurl?'
    live_room_init_api_url = (
        'https://api.live.bilibili.com/room/v1/Room/room_init?id={}'
    )
    live_room_info_api_url = (
        'https://api.live.bilibili.com/room/v1/Room/get_info?room_id={}'
    )

    SEC1 = '1c15888dc316e05a15fdd0a02ed6584f'
    SEC2 = '9b288147e5474dd2aa67085f716c560d'
    stream_types = [
        {'id': 'hdflv'},
        {'id': 'flv720'},
        {'id': 'flv'},
        {'id': 'hdmp4'},
        {'id': 'mp4'},
        {'id': 'live'},
        {'id': 'vc'},
    ]
    fmt2qlt = dict(hdflv=4, flv=3, hdmp4=2, mp4=1)

    @staticmethod
    def bilibili_stream_type(urls):
        url = urls[0]
        if 'hd.flv' in url or '-112.flv' in url:
            return 'hdflv', 'flv'
        if '-64.flv' in url:
            return 'flv720', 'flv'
        if '.flv' in url:
            return 'flv', 'flv'
        if 'hd.mp4' in url or '-48.mp4' in url:
            return 'hdmp4', 'mp4'
        if '.mp4' in url:
            return 'mp4', 'mp4'
        raise Exception('Unknown stream type')

    def api_req(self, cid, quality, bangumi, bangumi_movie=False, **kwargs):
        ts = str(int(time.time()))
        if not bangumi:
            params_str = 'cid={}&player=1&quality={}&ts={}'.format(
                cid, quality, ts
            )
            chksum = hashlib.md5(
                bytes(params_str+self.SEC1, 'utf8')
            ).hexdigest()
            api_url = self.api_url + params_str + '&sign=' + chksum
        else:
            mod = 'movie' if bangumi_movie else 'bangumi'
            params_str = 'cid={}&module={}&player=1&quality={}&ts={}'.format(
                cid, mod, quality, ts
            )
            chksum = hashlib.md5(
                bytes(params_str+self.SEC2, 'utf8')
            ).hexdigest()
            api_url = self.bangumi_api_url + params_str + '&sign=' + chksum

        xml_str = get_content(api_url, headers={
            'Referer': self.url,
            'User-Agent': FAKE_HEADERS['User-Agent'],
        })
        return xml_str

    def parse_bili_xml(self, xml_str):
        urls_list = []
        total_size = 0
        doc = parseString(xml_str.encode('utf8'))
        durls = doc.getElementsByTagName('durl')
        for durl in durls:
            size = durl.getElementsByTagName('size')[0]
            total_size += int(size.firstChild.nodeValue)
            url = durl.getElementsByTagName('url')[0]
            urls_list.append(url.firstChild.nodeValue)
        if not urls_list:
            return
        stream_type, container = self.bilibili_stream_type(urls_list)
        if stream_type not in self.streams:
            self.streams[stream_type] = {}
            self.streams[stream_type]['src'] = urls_list
            self.streams[stream_type]['size'] = total_size
            self.streams[stream_type]['container'] = container

    def download_by_vid(self, cid, bangumi, **kwargs):
        stream_id = kwargs.get('stream_id')
        # guard here. if stream_id invalid, fallback as not stream_id

        info_only = kwargs.get('info_only')
        for qlt in range(4, -1, -1):
            api_xml = self.api_req(cid, qlt, bangumi, **kwargs)
            self.parse_bili_xml(api_xml)
        if not info_only or stream_id:
            self.danmuku = get_danmuku_xml(cid)

    def prepare(self, **kwargs):
        if socket.getdefaulttimeout() == 600:  # no timeout specified
            socket.setdefaulttimeout(2)  # fail fast, very speedy!

        # handle 'watchlater' URLs
        if '/watchlater/' in self.url:
            aid = match1(self.url, r'av(\d+)')
            self.url = 'https://www.bilibili.com/video/av{}/'.format(aid)
        self.ua = FAKE_HEADERS['User-Agent']
        if 'bangumi' not in self.url:
            # bangumi redirect will miss fragment argument here
            # http://bangumi.bilibili.com/anime/21542/play#173286 ->
            # https://www.bilibili.com/bangumi/play/ss21542
            # It should be https://www.bilibili.com/bangumi/play/ss21542#173286
            self.url = url_locations([self.url])[0]

        frag = urllib.parse.urlparse(self.url).fragment
        # http://www.bilibili.com/video/av3141144/index_2.html#page=3
        if frag:
            page = match1(frag, r'page=(\d+)')
            if page:
                aid = match1(self.url, r'av(\d+)')
                self.url = (
                    'https://www.bilibili.com/video/av{}/index_{}.html'.format(
                        aid, page
                    )
                )

        # handle bangumi url like this
        # http://bangumi.bilibili.com/anime/21542/play#173286
        # https://www.bilibili.com/bangumi/play/ss21542#173286
        # https://www.bilibili.com/bangumi/play/ep173286
        bangumi_ep_id = match1(self.url, r'/anime/\d+/play#(\d+)') or \
            match1(self.url, r'/bangumi/play/ss\d+#(\d+)')
        if bangumi_ep_id:
            self.url = 'https://www.bilibili.com/bangumi/play/ep{}'.format(
                bangumi_ep_id
            )

        self.referer = self.url
        self.page = get_content(self.url)
        self.parser = get_parser(self.page)
        if self.parser.h1:
            self.title = self.parser.h1.text.strip()
        else:
            # Some movie page got no h1 tag
            self.title = self.parser.find(
                'meta', property='og:title'
            )['content']
        if 'subtitle' in kwargs:
            subtitle = kwargs['subtitle']
            self.title = '{} {}'.format(self.title, subtitle)

        if 'live.bilibili.com' in self.url:
            self.live_entry(**kwargs)
        elif 'vc.bilibili.com' in self.url:
            self.vc_entry(**kwargs)
        else:
            # bangumi, movie use this entry too
            self.entry(**kwargs)

    def entry(self, **kwargs):
        # tencent player
        tc_flashvars = match1(
            self.page, r'"bili-cid=\d+&bili-aid=\d+&vid=([^"]+)"'
        )
        if tc_flashvars:
            self.out = True
            qq_download_by_vid(
                tc_flashvars, self.title, output_dir=kwargs['output_dir'],
                merge=kwargs['merge'], info_only=kwargs['info_only']
            )
            return

        has_plist = r'<option' in self.page
        if has_plist and match1(self.url, 'index_(\d+).html') is None:
            log.w(
                'This page contains a playlist. (use --playlist to download '
                'all videos.)'
            )

        cid = match1(self.page, r'cid=(\d+)') \
            or match1(self.page, r'"cid":(\d+)')
        if cid:
            self.download_by_vid(
                cid, 'bangumi' in self.url, **kwargs
            )
        else:
            # flashvars?
            flashvars = match1(self.page, r'flashvars="([^"]+)"')
            if flashvars is None:
                raise Exception('Unsupported page {}'.format(self.url))
            param = flashvars.split('&')[0]
            t, cid = param.split('=')
            t = t.strip()
            cid = cid.strip()
            if t == 'vid':
                sina_download_by_vid(
                    cid, self.title, output_dir=kwargs['output_dir'],
                    merge=kwargs['merge'], info_only=kwargs['info_only']
                )
            elif t == 'ykid':
                youku_download_by_vid(
                    cid, self.title, output_dir=kwargs['output_dir'],
                    merge=kwargs['merge'], info_only=kwargs['info_only']
                )
            else:
                raise NotImplementedError(
                    'Unknown flashvars {}'.format(flashvars)
                )
            return

    def live_entry(self, **kwargs):
        # Extract room ID from the short display ID (seen in the room
        # URL). The room ID is usually the same as the short ID, but not
        # always; case in point: https://live.bilibili.com/48, with 48
        # as the short ID and 63727 as the actual ID.
        room_short_id = match1(
            self.url, r'live.bilibili.com/([^?]+)'
        )
        room_init_api_response = json.loads(get_content(
            self.live_room_init_api_url.format(room_short_id)
        ))
        self.room_id = room_init_api_response['data']['room_id']

        room_info_api_response = json.loads(get_content(
            self.live_room_info_api_url.format(self.room_id)
        ))
        self.title = room_info_api_response['data']['title']

        api_url = self.live_api.format(self.room_id)
        json_data = json.loads(get_content(api_url))
        urls = [json_data['durl'][0]['url']]

        self.streams['live'] = {}
        self.streams['live']['src'] = urls
        self.streams['live']['container'] = 'flv'
        self.streams['live']['size'] = 0

    def vc_entry(self, **kwargs):
        vc_id = match1(self.url, r'video/(\d+)') \
            or match1(self.url, r'vcdetail\?vc=(\d+)')
        if not vc_id:
            log.wtf('Unknown url pattern')
        endpoint = (
            'https://api.vc.bilibili.com/clip/v1/video/detail?video_id={}'
            '&need_playurl=1'.format(vc_id)
        )
        vc_meta = json.loads(get_content(endpoint, headers=FAKE_HEADERS))
        if vc_meta['code'] != 0:
            log.wtf('{}\n{}'.format(vc_meta['msg'], vc_meta['message']))
        item = vc_meta['data']['item']
        self.title = item['description']

        self.streams['vc'] = {}
        self.streams['vc']['src'] = [item['video_playurl']]
        self.streams['vc']['container'] = 'mp4'
        self.streams['vc']['size'] = int(item['video_size'])


def collect_bangumi_urls(json_data):
    eps = json_data['episodes'][::-1]
    return [ep['webplay_url'] for ep in eps]


def get_bangumi_info(bangumi_id):
    BASE_URL = 'https://bangumi.bilibili.com/jsonp/seasoninfo/'
    long_epoch = int(time.time() * 1000)
    req_url = '{}{}.ver?callback=seasonListCallback&jsonp=jsonp&_={}'.format(
        BASE_URL, bangumi_id, str(long_epoch)
    )
    season_data = get_content(req_url)
    season_data = season_data[len('seasonListCallback('):]
    season_data = season_data[: -1 * len(');')]
    json_data = json.loads(season_data)
    return json_data['result']


def get_danmuku_xml(cid):
    return get_content('https://comment.bilibili.com/{}.xml'.format(cid))


site = Bilibili()


def bilibili_download_playlist_by_url(url, **kwargs):
    url = url_locations([url])[0]
    if 'live.bilibili' in url:
        site.download_by_url(url)
    elif 'bangumi.bilibili' in url:
        bangumi_id = match1(url, r'(\d+)')
        bangumi_data = get_bangumi_info(bangumi_id)
        ep_urls = collect_bangumi_urls(bangumi_data)
        for ep_url in ep_urls:
            Bilibili().download_by_url(ep_url, **kwargs)
    else:
        aid = match1(url, r'av(\d+)')
        if aid:
            # normal playlist
            # https://www.bilibili.com/video/av16907446/
            page_list = json.loads(get_content(
                'https://www.bilibili.com/widget/getPageList?aid={}'.format(
                    aid
                )
            ))
            page_cnt = len(page_list)
            for no in range(1, page_cnt+1):
                page_url = (
                    'https://www.bilibili.com/video/av{}/index_{}.html'.format(
                        aid, no
                    )
                )
                subtitle = page_list[no-1]['pagename']
                # 循环里面不能用同一个实例，self.streams 不会改变的
                # 它里面始终存的是第一个地址的最高清晰度的 url
                # parse_bili_xml L107
                Bilibili().download_by_url(
                    page_url, subtitle=subtitle, **kwargs
                )
        else:
            # tv playlist
            # https://www.bilibili.com/bangumi/play/ep196751/
            page = get_content(url)
            ep_data = json.loads(match1(
                page, r'window.__INITIAL_STATE__=(.+?);'
            ))
            for ep in ep_data['epList']:
                ep_url = 'https://www.bilibili.com/bangumi/play/ep{}'.format(
                    ep['ep_id']
                )
                Bilibili().download_by_url(ep_url, **kwargs)


download = bilibili_download = site.download_by_url
download_playlist = bilibili_download_playlist_by_url



#!/usr/bin/env python

import re
import json

from lulu.common import (
    match1,
    print_info,
    get_content,
    download_url_ffmpeg,
    playlist_not_supported,
)


__all__ = ['metacafe_download']
site_info = 'Metacafe metacafe.com'


def metacafe_download(url, info_only=False, **kwargs):
    if re.match(r'http://www.metacafe.com/watch/\w+', url):
        html = get_content(url)
        title = match1(html, r'<meta property="og:title" content="([^"]*)"')

        data = match1(
            html,
            r"<script type='text/json' id='json_video_data'>(.+)</script>"
        )
        data = json.loads(data)
        m3u8_url = data['sources'][0]['src']
        print_info(
            site_info, title, 'm3u8', 0, m3u8_url=m3u8_url, m3u8_type='master'
        )
        if not info_only:
            download_url_ffmpeg(m3u8_url, title, 'mp4', **kwargs)


download = metacafe_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import json

from lulu.common import (
    match1,
    url_info,
    get_content,
)
from lulu.config import FAKE_HEADERS
from lulu.extractor import VideoExtractor


class Bigthink(VideoExtractor):
    name = 'Bigthink bigthink.com'

    # this is just a sample. Will make it in prepare()
    stream_types = [
        # {'id': '1080'},
        # {'id': '720'},
        # {'id': '360'},
        # {'id': '288'},
        # {'id': '190'},
        # {'id': '180'},
    ]

    @staticmethod
    def get_streams_by_id(account_number, video_id):
        """
        int, int->list

        Get the height of the videos.

        Since brightcove is using 3 kinds of links: rtmp, http and https,
        we will be using the HTTPS one to make it secure.

        If somehow akamaihd.net is blocked by the Great Fucking Wall,
        change the "startswith https" to http.
        """
        endpoint = (
            'https://edge.api.brightcove.com/playback/v1/accounts/'
            '{account_number}/videos/{video_id}'.format(
                account_number=account_number, video_id=video_id
            )
        )
        fake_header_id = FAKE_HEADERS.copy()
        # is this somehow related to the time? Magic....
        fake_header_id['Accept'] = (
            'application/json;pk=BCpkADawqM1cc6wmJQC2tvoXZt4mrB7bFfi6zGt9QnOzp'
            'rPZcGLE9OMGJwspQwKfuFYuCjAAJ53JdjI8zGFx1ll4rxhYJ255AXH1BQ10rnm34w'
            'eknpfG-sippyQ'
        )

        html = get_content(endpoint, headers=fake_header_id)
        html_json = json.loads(html)

        link_list = []
        for i in html_json['sources']:
            if 'src' in i and i.get('container') == 'MP4':  # to avoid KeyError
                if i['src'].startswith('https'):
                    link_list.append((str(i['height']), i['src']))

        return link_list

    def prepare(self, **kwargs):
        html = get_content(self.url)
        self.title = match1(
            html, r'<meta property="og:title" content="([^"]*)"'
        )
        account_number = match1(html, r'data-account="(\d+)"')
        video_id = match1(html, r'data-brightcove-id="(\d+)"')
        assert account_number, video_id
        link_list = self.get_streams_by_id(account_number, video_id)
        for i in link_list:
            self.stream_types.append({'id': str(i[0])})
            self.streams[i[0]] = {'url': i[1]}

    def extract(self, **kwargs):
        for i in self.streams:
            s = self.streams[i]
            _, s['container'], s['size'] = url_info(s['url'])
            s['src'] = [s['url']]


site = Bigthink()
download = site.download_by_url



#!/usr/bin/env python

import re
import http.client
from time import time
from copy import copy
from random import random
import xml.etree.ElementTree as ET

from lulu.common import (
    match1,
    print_info,
    get_content,
    download_urls,
)
from lulu.config import FAKE_HEADERS


__all__ = ['ucas_download', 'ucas_download_single', 'ucas_download_playlist']
site_info = '中国科学院大学 ucas.ac.cn'


"""
Do not replace http.client with get_content
for UCAS's server is not correctly returning data!
"""


def dictify(r, root=True):
    """http://stackoverflow.com/a/30923963/2946714
    """
    if root:
        return {r.tag: dictify(r, False)}
    d = copy(r.attrib)
    if r.text:
        d['_text'] = r.text
    for x in r.findall('./*'):
        if x.tag not in d:
            d[x.tag] = []
        d[x.tag].append(dictify(x, False))
    return d


def _get_video_query_url(resourceID):
    # has to be like this
    headers = FAKE_HEADERS.copy()
    headers.update({
        'DNT': '1',
        'Referer': 'http://v.ucas.ac.cn/',
        'Connection': 'keep-alive',
    })
    conn = http.client.HTTPConnection('210.76.211.10')

    conn.request(
        'GET',
        '/vplus/remote.do?method=query2&loginname=videocas&pwd=af1c7a4c5f77'
        'f790722f7cae474c37e281203765d423a23b&resource=%5B%7B%22resourceID%2'
        '2%3A%22{}%22%2C%22on%22%3A1%2C%22time%22%3A600%2C%22eid%22%3A100%2C'
        '%22w%22%3A800%2C%22h%22%3A600%7D%5D&timeStamp={}'.format(
            resourceID, str(int(time()))
        ),
        headers=headers
    )
    res = conn.getresponse()
    data = res.read()

    info = data.decode("utf-8")
    return match1(info, r'video":"(.+)"')


def _get_virtualPath(video_query_url):
    # getResourceJsCode2
    html = get_content(video_query_url)
    return match1(
        html, r"function\s+getVirtualPath\(\)\s+{\s+return\s+'(\w+)'"
    )


def _get_video_list(resourceID):
    conn = http.client.HTTPConnection('210.76.211.10')

    conn.request(
        'GET',
        '/vplus/member/resource.do?isyulan=0&method=queryFlashXmlByResourceId'
        '&resourceId={resourceID}&randoms={randoms}'.format(
            resourceID=resourceID, randoms=random()
        )
    )
    res = conn.getresponse()
    data = res.read()

    video_xml = data.decode('utf-8')

    root = ET.fromstring(video_xml.split('___!!!___')[0])

    r = dictify(root)

    huge_list = []
    # main
    huge_list.append(
        [i['value'] for i in sorted(
            r['video']['mainUrl'][0]['_flv'][0]['part'][0]['video'],
            key=lambda k: int(k['index'])
        )]
    )

    # sub
    if '_flv' in r['video']['subUrl'][0]:
        huge_list.append([i['value'] for i in sorted(
            r['video']['subUrl'][0]['_flv'][0]['part'][0]['video'],
            key=lambda k: int(k['index'])
        )])

    return huge_list


def _ucas_get_url_lists_by_resourceID(resourceID):
    video_query_url = _get_video_query_url(resourceID)
    assert video_query_url != '', 'Cannot find video GUID!'

    virtualPath = _get_virtualPath(video_query_url)
    assert virtualPath != '', 'Cannot find virtualPath!'

    url_lists = _get_video_list(resourceID)
    assert url_lists, 'Cannot find any URL to download!'

    # make real url
    # credit to a mate in UCAS
    for video_type_id, video_urls in enumerate(url_lists):
        for k, path in enumerate(video_urls):
            url_lists[video_type_id][k] = (
                'http://210.76.211.10/vplus/member/resource.do?virtualPath='
                '{virtualPath}&method=getImgByStream&imgPath={path}'.format(
                    virtualPath=virtualPath, path=path
                )
            )
    return url_lists


def ucas_download_single(url, info_only=False, **kwargs):
    '''video page
    '''
    html = get_content(url)
    # resourceID is UUID
    resourceID = re.findall(
        r'resourceID":"([0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-'
        r'[0-9a-f]{12})',
        html
    )[0]
    assert resourceID != '', 'Cannot find resourceID!'

    title = match1(html, r'<div class="bc-h">(.+)</div>')
    url_lists = _ucas_get_url_lists_by_resourceID(resourceID)
    assert url_lists, 'Cannot find any URL of such class!'

    for k, part in enumerate(url_lists):
        part_title = '{}_{}'.format(title, str(k))
        print_info(site_info, part_title, 'flv', 0)
        if not info_only:
            download_urls(part, part_title, 'flv', total_size=None, **kwargs)


def ucas_download_playlist(url, info_only=False, **kwargs):
    '''course page
    '''
    html = get_content(url)

    parts = re.findall(r'(getplaytitle.do\?.+)"', html)
    assert parts, 'No part found!'

    for part_path in parts:
        ucas_download(
            'http://v.ucas.ac.cn/course/{}'.format(part_path),
            info_only=info_only, **kwargs
        )


def ucas_download(url, info_only=False, **kwargs):
    if 'classid=' in url and 'getplaytitle.do' in url:
        ucas_download_single(url, info_only=info_only, **kwargs)
    elif 'CourseIndex.do' in url:
        ucas_download_playlist(url, info_only=info_only, **kwargs)


download = ucas_download
download_playlist = ucas_download_playlist



#!/usr/bin/env python
import time
import random
from xml.dom import minidom

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)


__all__ = ['tucao_download']
site_info = '吐槽弹幕网 tucao.tv'


# possible raw list types
# 1. <li>type=tudou&vid=199687639</li>
# 2. <li>type=tudou&vid=199506910|</li>
# 3. <li>type=video&file=http://xiaoshen140731.qiniudn.com/lovestage04.flv|</li>  # noqa
# 4 may ? <li>type=video&file=http://xiaoshen140731.qiniudn.com/lovestage04.flv|xx**type=&vid=?</li>  # noqa
# 5. <li>type=tudou&vid=200003098|07**type=tudou&vid=200000350|08</li>
# 6. <li>vid=49454694&type=sina|</li>
# 7. <li>type=189&vid=513031813243909|</li>
# re_pattern=re.compile(r"(type=(.+?)&(vid|file)=(.*?))[\|<]")


def tucao_single_download(type_link, title, info_only=False, **kwargs):
    if 'file' in type_link:
        url = type_link[type_link.find('file=')+5:]
        vtype, ext, size = url_info(url)
        print_info(site_info, title, vtype, size)
        if not info_only:
            download_urls([url], title, ext, size, **kwargs)
    # fix for 189 video source, see raw list types 7
    elif '189' in type_link:
        vid = match1(type_link, r'vid=(\d+)')
        assert vid, 'vid not exsits'
        url = 'http://api.tucao.tv/api/down/{}'.format(vid)
        vtype, ext, size = url_info(url)
        print_info(site_info, title, vtype, size)
        if not info_only:
            download_urls([url], title, ext, size, **kwargs)
    else:
        u = (
            'http://www.tucao.tv/api/playurl.php?{}&key=tucao{:07x}.'
            'cc&r={}'.format(
                type_link, random.getrandbits(28), int(time.time()*1000)
            )
        )
        xml = minidom.parseString(get_content(u))
        urls = []
        size = 0
        for i in xml.getElementsByTagName('url'):
            urls.append(i.firstChild.nodeValue)
            vtype, ext, _size = url_info(i.firstChild.nodeValue)
            size += _size
        print_info(site_info, title, vtype, size)
        if not info_only:
            download_urls(urls, title, ext, size, **kwargs)


def tucao_download(url, info_only=False, **kwargs):
    html = get_content(url)
    title = match1(html, r'<h1 class="show_title">(.*?)<\w')
    # fix for raw list that vid goes before type, see raw list types 6
    raw_list = match1(html, r'<li>\s*(type=.+?|vid=.+?)</li>')
    raw_l = raw_list.split("**")
    if len(raw_l) == 1:
        format_link = raw_l[0][:-1] if raw_l[0].endswith('|') else raw_l[0]
        tucao_single_download(format_link, title, info_only, **kwargs)
    else:
        for i in raw_l:
            format_link, sub_title = i.split('|')
            tucao_single_download(
                format_link, '{}-{}'.format(title, sub_title), info_only,
                **kwargs
            )


download = tucao_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import json

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
)


__all__ = ['bandcamp_download']
site_info = 'Bandcamp bandcamp.com'


def bandcamp_download(
    url, output_dir='.', merge=True, info_only=False, **kwargs
):
    html = get_content(url)
    trackinfo = json.loads(
        match1(html, r'trackinfo:\s+(\[.+\]),')
    )
    for track in trackinfo:
        track_num = track['track_num']
        title = '{}. {}'.format(track_num, track['title'])
        file_url = track['file']['mp3-128']
        mime, ext, size = url_info(file_url)

        print_info(site_info, title, mime, size)
        if not info_only:
            download_urls(
                [file_url], title, ext, size, output_dir, merge=merge,
                **kwargs
            )


download = bandcamp_download
download_playlist = bandcamp_download



#!/usr/bin/env python

import re
import json
import time
import hashlib


from lulu.util import log
from lulu.common import (
    match1,
    print_info,
    get_content,
    download_urls,
    download_url_ffmpeg,
    general_m3u8_extractor,
    playlist_not_supported,
)


__all__ = ['douyutv_download']
site_info = '斗鱼 douyu.com'


def douyutv_video_download(
    url, output_dir='.', merge=True, info_only=False, **kwargs
):
    ep = 'http://vmobile.douyu.com/video/getInfo?vid='
    patt = r'show/([0-9A-Za-z]+)'
    title_patt = r'<h1>(.+?)</h1>'

    hit = re.search(patt, url)
    if hit is None:
        log.wtf('Unknown url pattern')
    vid = hit.group(1)

    page = get_content(url)
    hit = re.search(title_patt, page)
    if hit is None:
        title = vid
    else:
        title = hit.group(1)

    meta = json.loads(get_content(ep + vid))
    if meta['error'] != 0:
        log.wtf('Error from API server')
    m3u8_url = meta['data']['video_url']
    print_info(site_info, title, 'm3u8', 0, m3u8_url=m3u8_url)
    if not info_only:
        urls = general_m3u8_extractor(m3u8_url)
        download_urls(
            urls, title, 'ts', 0, output_dir=output_dir, merge=merge, **kwargs
        )


def douyutv_download(
    url, output_dir='.', merge=True, info_only=False, **kwargs
):
    if 'v.douyu.com/show/' in url:
        douyutv_video_download(
            url, output_dir=output_dir, merge=merge, info_only=info_only,
            **kwargs
        )
        return
    url = re.sub(r'[\w.]*douyu.com', 'm.douyu.com', url)
    html = get_content(url)
    room_id_patt = r'room_id\s*:\s*(\d+),'
    room_id = match1(html, room_id_patt)
    if room_id == '0':
        room_id = url[url.rfind('/')+1:]

    api_url = 'http://www.douyutv.com/api/v1/'
    args = 'room/{}?aid=wp&client_sys=wp&time={}'.format(
        room_id, int(time.time())
    )
    auth_md5 = (args + 'zNzMV1y4EMxOHS6I5WKm').encode('utf-8')
    auth_str = hashlib.md5(auth_md5).hexdigest()
    json_request_url = '{}{}&auth={}'.format(api_url, args, auth_str)

    content = get_content(json_request_url)
    json_content = json.loads(content)
    data = json_content['data']
    server_status = json_content.get('error', 0)
    if server_status is not 0:
        raise ValueError('Server returned error: {}'.format(server_status))

    title = data.get('room_name')
    show_status = data.get('show_status')
    if show_status is not '1':
        raise ValueError('The live stream is not online! (Errno: {})'.format(
            server_status
        ))

    real_url = '{}/{}'.format(
        data.get('rtmp_url'), data.get('rtmp_live')
    )
    print_info(site_info, title, 'flv', float('inf'))
    if not info_only:
        download_url_ffmpeg(
            real_url, title, 'flv', None, output_dir=output_dir,
            merge=merge
        )


download = douyutv_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import re
import os
import json
import urllib.parse as urlparse

from lulu.common import (
    match1,
    url_size,
    print_info,
    get_content,
    download_urls,
    playlist_not_supported,
)


__all__ = ['huaban_download']
site_info = '花瓣 (Huaban)'
LIMIT = 100


class Board:
    def __init__(self, title, pins):
        self.title = title
        self.pins = pins
        self.pin_count = len(pins)


class Pin:
    host = 'http://img.hb.aicdn.com/'

    def __init__(self, pin_json):
        img_file = pin_json['file']
        self.id = str(pin_json['pin_id'])
        self.url = urlparse.urljoin(self.host, img_file['key'])
        self.ext = img_file['type'].split('/')[-1]


def construct_url(url, **params):
    param_str = urlparse.urlencode(params)
    return url + '?' + param_str


def extract_json_data(url, **params):
    url = construct_url(url, **params)
    html = get_content(url)
    json_string = match1(html, r'app.page\["board"\] = (.*?});')
    json_data = json.loads(json_string)
    return json_data


def extract_board_data(url):
    json_data = extract_json_data(url, limit=LIMIT)
    pin_list = json_data['pins']
    title = json_data['title']
    pin_count = json_data['pin_count']
    pin_count -= len(pin_list)

    while pin_count > 0:
        json_data = extract_json_data(
            url, max=pin_list[-1]['pin_id'], limit=LIMIT
        )
        pins = json_data['pins']
        pin_list += pins
        pin_count -= len(pins)

    return Board(title, list(map(Pin, pin_list)))


def huaban_download_board(url, output_dir, **kwargs):
    board = extract_board_data(url)
    output_dir = os.path.join(output_dir, board.title)
    print_info(site_info, board.title, 'jpg', 0)
    if not kwargs['info_only']:
        for pin in board.pins:
            download_urls(
                [pin.url], pin.id, pin.ext, url_size(pin.url),
                output_dir=output_dir, faker=True, **kwargs
            )


def huaban_download(url, output_dir='.', **kwargs):
    if re.match(r'http://huaban\.com/boards/\d+/', url):
        huaban_download_board(url, output_dir, **kwargs)
    else:
        print('Only board (画板) pages are supported currently')
        print('For example: http://huaban.com/boards/16687763/')


download = huaban_download
download_playlist = playlist_not_supported(site_info)



#!/usr/bin/env python

import logging
import os.path
import subprocess
from subprocess import DEVNULL

from lulu.util.strings import parameterize


def get_usable_ffmpeg(cmd):
    try:
        p = subprocess.Popen(
            [cmd, '-version'], stdin=DEVNULL, stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        out, err = p.communicate()
        vers = str(out, 'utf-8').split('\n')[0].split()
        assert (vers[0] == 'ffmpeg' and vers[2][0] > '0') or \
            (vers[0] == 'avconv')
        # set version to 1.0 for nightly build and print warning
        try:
            version = [int(i) for i in vers[2].split('.')]
        except Exception:
            print('It seems that your ffmpeg is a nightly build.')
            print('Please switch to the latest stable if merging failed.')
            version = [1, 0]
        return cmd, 'ffprobe', version
    except Exception:
        return None


FFMPEG, FFPROBE, FFMPEG_VERSION = get_usable_ffmpeg('ffmpeg') or \
    get_usable_ffmpeg('avconv') or (None, None, None)

if logging.getLogger().isEnabledFor(logging.DEBUG):
    LOGLEVEL = ['-loglevel', 'info']
    STDIN = None
else:
    LOGLEVEL = ['-loglevel', 'quiet']
    STDIN = DEVNULL


def has_ffmpeg_installed():
    return FFMPEG is not None


# Given a list of segments and the output path, generates the concat
# list and returns the path to the concat list.
def generate_concat_list(files, output):
    concat_list_path = output + '.txt'
    concat_list_dir = os.path.dirname(concat_list_path)
    with open(concat_list_path, 'w', encoding='utf-8') as concat_list:
        for file in files:
            if os.path.isfile(file):
                relpath = os.path.relpath(file, start=concat_list_dir)
                concat_list.write('file %s\n' % parameterize(relpath))
    return concat_list_path


def ffmpeg_concat_av(files, output, ext):
    print('Merging video parts... ', end="", flush=True)
    params = [FFMPEG] + LOGLEVEL
    for file in files:
        if os.path.isfile(file):
            params.extend(['-i', file])
    params.extend(['-c:v', 'copy'])
    if ext == 'mp4':
        params.extend(['-c:a', 'aac'])
    elif ext == 'webm':
        params.extend(['-c:a', 'vorbis'])
    params.extend(['-strict', 'experimental'])
    params.append(output)
    return subprocess.call(params, stdin=STDIN)


def ffmpeg_convert_ts_to_mkv(files, output='output.mkv'):
    for file in files:
        if os.path.isfile(file):
            params = [FFMPEG] + LOGLEVEL
            params.extend(['-y', '-i', file, output])
            subprocess.call(params, stdin=STDIN)
    return


def ffmpeg_concat_mp4_to_mpg(files, output='output.mpg'):
    # Use concat demuxer on FFmpeg >= 1.1
    if FFMPEG == 'ffmpeg' and (
        FFMPEG_VERSION[0] >= 2 or (
            FFMPEG_VERSION[0] == 1 and FFMPEG_VERSION[1] >= 1
        )
    ):
        concat_list = generate_concat_list(files, output)
        params = [FFMPEG] + LOGLEVEL + [
            '-y', '-f', 'concat', '-safe', '-1', '-i', concat_list, '-c',
            'copy', output
        ]
        if subprocess.call(params, stdin=STDIN) == 0:
            os.remove(output + '.txt')
            return True
        else:
            raise

    for file in files:
        if os.path.isfile(file):
            params = [FFMPEG] + LOGLEVEL + ['-y', '-i']
            params.extend([file, file + '.mpg'])
            subprocess.call(params, stdin=STDIN)

    inputs = [open(file + '.mpg', 'rb') for file in files]
    with open(output + '.mpg', 'wb') as o:
        for input in inputs:
            o.write(input.read())

    params = [FFMPEG] + LOGLEVEL + ['-y', '-i']
    params.append(output + '.mpg')
    params += ['-vcodec', 'copy', '-acodec', 'copy']
    params.append(output)

    if subprocess.call(params, stdin=STDIN) == 0:
        for file in files:
            os.remove(file + '.mpg')
        os.remove(output + '.mpg')
        return True
    else:
        raise


def ffmpeg_concat_ts_to_mkv(files, output='output.mkv'):
    print('Merging video parts... ', end='', flush=True)
    params = [FFMPEG] + LOGLEVEL + ['-isync', '-y', '-i']
    params.append('concat:')
    for file in files:
        if os.path.isfile(file):
            params[-1] += file + '|'
    params += ['-f', 'matroska', '-c', 'copy', output]

    try:
        if subprocess.call(params, stdin=STDIN) == 0:
            return True
        else:
            return False
    except Exception:
        return False


def ffmpeg_concat_flv_to_mp4(files, output='output.mp4'):
    print('Merging video parts... ', end='', flush=True)
    # Use concat demuxer on FFmpeg >= 1.1
    if FFMPEG == 'ffmpeg' and (
        FFMPEG_VERSION[0] >= 2 or (
            FFMPEG_VERSION[0] == 1 and FFMPEG_VERSION[1] >= 1
        )
    ):
        concat_list = generate_concat_list(files, output)
        params = [FFMPEG] + LOGLEVEL + [
            '-y', '-f', 'concat', '-safe', '-1', '-i', concat_list, '-c',
            'copy', '-bsf:a', 'aac_adtstoasc', output
        ]
        subprocess.check_call(params, stdin=STDIN)
        os.remove(output + '.txt')
        return True

    for file in files:
        if os.path.isfile(file):
            params = [FFMPEG] + LOGLEVEL + ['-y', '-i']
            params.append(file)
            params += [
                '-map', '0', '-c', 'copy', '-f', 'mpegts', '-bsf:v',
                'h264_mp4toannexb'
            ]
            params.append(file + '.ts')

            subprocess.call(params, stdin=STDIN)

    params = [FFMPEG] + LOGLEVEL + ['-y', '-i']
    params.append('concat:')
    for file in files:
        f = file + '.ts'
        if os.path.isfile(f):
            params[-1] += f + '|'
    if FFMPEG == 'avconv':
        params += ['-c', 'copy', output]
    else:
        params += ['-c', 'copy', '-absf', 'aac_adtstoasc', output]

    if subprocess.call(params, stdin=STDIN) == 0:
        for file in files:
            os.remove(file + '.ts')
        return True
    else:
        raise


def ffmpeg_concat_mp4_to_mp4(files, output='output.mp4'):
    print('Merging video parts... ', end='', flush=True)
    # Use concat demuxer on FFmpeg >= 1.1
    if FFMPEG == 'ffmpeg' and (
        FFMPEG_VERSION[0] >= 2 or (
            FFMPEG_VERSION[0] == 1 and FFMPEG_VERSION[1] >= 1
        )
    ):
        concat_list = generate_concat_list(files, output)
        params = [FFMPEG] + LOGLEVEL + [
            '-y', '-f', 'concat', '-safe', '-1', '-i', concat_list, '-c',
            'copy', '-bsf:a', 'aac_adtstoasc', output
        ]
        subprocess.check_call(params, stdin=STDIN)
        os.remove(output + '.txt')
        return True

    for file in files:
        if os.path.isfile(file):
            params = [FFMPEG] + LOGLEVEL + ['-y', '-i']
            params.append(file)
            params += [
                '-c', 'copy', '-f', 'mpegts', '-bsf:v', 'h264_mp4toannexb'
            ]
            params.append(file + '.ts')

            subprocess.call(params, stdin=STDIN)

    params = [FFMPEG] + LOGLEVEL + ['-y', '-i']
    params.append('concat:')
    for file in files:
        f = file + '.ts'
        if os.path.isfile(f):
            params[-1] += f + '|'
    if FFMPEG == 'avconv':
        params += ['-c', 'copy', output]
    else:
        params += ['-c', 'copy', '-absf', 'aac_adtstoasc', output]

    subprocess.check_call(params, stdin=STDIN)
    for file in files:
        os.remove(file + '.ts')
    return True


def ffmpeg_download_stream(
    files, title, ext, params={}, output_dir='.', stream=True, **kwargs
):
    """str, str->True
    WARNING: NOT THE SAME PARMS AS OTHER FUNCTIONS!!!!!!
    You can basicly download anything with this function
    but better leave it alone with
    """
    # https://ffmpeg.org/ffmpeg.html#Main-options
    output = '{}.{}'.format(title, ext)

    if not (output_dir == '.'):
        output = '{}/{}'.format(output_dir, output)

    print(
        'Downloading streaming content with FFmpeg, press q to stop '
        'recording...'
    )
    if stream:
        ffmpeg_params = [FFMPEG] + ['-y', '-re', '-i']
    else:
        ffmpeg_params = [FFMPEG] + ['-y', '-i']
    ffmpeg_params.append(files)  # not the same here!!!!

    if FFMPEG == 'avconv':  # who cares?
        ffmpeg_params += ['-c', 'copy', output]
    elif kwargs.get('override'):
        pass
    else:
        ffmpeg_params += ['-c', 'copy', '-bsf:a', 'aac_adtstoasc']

    if params is not None:
        if len(params) > 0:
            for k, v in params.items():
                ffmpeg_params.append(k)
                ffmpeg_params.append(v)

    ffmpeg_params.append(output)

    print(' '.join(ffmpeg_params))

    try:
        a = subprocess.Popen(ffmpeg_params, stdin=subprocess.PIPE)
        a.communicate()
    except KeyboardInterrupt:
        try:
            a.stdin.write('q'.encode('utf-8'))
        except Exception:
            pass

    return True


def ffmpeg_concat_audio_and_video(files, output, ext):
    print('Merging video and audio parts... ', end='', flush=True)
    if has_ffmpeg_installed:
        params = [FFMPEG] + LOGLEVEL
        params.extend(['-f', 'concat'])
        params.extend(['-safe', '0'])
        for file in files:
            if os.path.isfile(file):
                params.extend(['-i', file])
        params.extend(['-c:v', 'copy'])
        params.extend(['-c:a', 'aac'])
        params.extend(['-strict', 'experimental'])
        params.append('{}.{}'.format(output, ext))
        return subprocess.call(params, stdin=STDIN)
    else:
        raise EnvironmentError('No ffmpeg found')


def ffprobe_get_media_duration(file):
    print('Getting {} duration'.format(file))
    params = [FFPROBE]
    params.extend(['-i', file])
    params.extend(['-show_entries', 'format=duration'])
    params.extend(['-v', 'quiet'])
    params.extend(['-of', 'csv=p=0'])
    return subprocess.check_output(
        params, stdin=STDIN, stderr=subprocess.STDOUT
    ).decode().strip()



#!/usr/bin/env python

import re
from urllib import parse

from lulu.common import (
    match1,
    url_info,
    print_info,
    get_content,
    download_urls,
)


__all__ = ['heavymusic_download']
site_info = 'Heavy Music Archive heavy-music.ru'


def heavymusic_download(url, info_only=False, **kwargs):
    html = get_content(url)
    tracks = re.findall(r'href="(online2\.php[^"]+)"', html)
    for track in tracks:
        band = match1(track, r'band=([^&]*)')
        album = match1(track, r'album=([^&]*)')
        title = match1(track, r'track=([^&]*)')
        file_url = (
            'http://www.heavy-music.ru/online2.php?band={}&album={}&'
            'track={}'.format(
                parse.quote(band), parse.quote(album), parse.quote(title)
            )
        )
        _, _, size = url_info(file_url)

        print_info(site_info, title, 'mp3', size)
        if not info_only:
            download_urls([file_url], title[:-4], 'mp3', size, **kwargs)


download = heavymusic_download
download_playlist = heavymusic_download



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import json
import time
from urllib import (
    parse,
    request,
)

from lulu.common import (
    match1,
    ungzip,
    cookies,
    url_info,
    print_info,
    get_content,
    post_content,
    download_urls,
    playlist_not_supported,
)
from lulu.config import FAKE_HEADERS
from lulu.util.parser import get_parser
from lulu.extractors.embed import embed_download


__all__ = ['baidu_download']
site_info = '百度 baidu.com'


def baidu_get_song_data(sid):
    data = json.loads(get_content(
        'http://music.baidu.com/data/music/fmlink?songIds={}'.format(sid)
    ))['data']

    if data['xcode'] != '':
        # inside china mainland
        return data['songList'][0]
    else:
        # outside china mainland
        return None


def baidu_download_song(sid, output_dir='.', merge=True, info_only=False):
    data = baidu_get_song_data(sid)
    if data is not None:
        url = data['songLink']
        title = data['songName']
        artist = data['artistName']
        # album = data['albumName']
        lrc = data['lrcLink']
        file_name = '{} - {}'.format(title, artist)
    else:
        html = get_content('http://music.baidu.com/song/{}'.format(sid))
        url = match1(html, r'data_url="([^"]+)"')
        title = match1(html, r'data_name="([^"]+)"')
        file_name = title

    _type, ext, size = url_info(url, faker=True)
    print_info(site_info, title, _type, size)
    if not info_only:
        download_urls(
            [url], file_name, ext, size, output_dir, merge=merge, faker=True
        )

    try:
        _type, ext, size = url_info(lrc, faker=True)
        print_info(site_info, title, _type, size)
        if not info_only:
            download_urls([lrc], file_name, ext, size, output_dir, faker=True)
    except Exception:
        pass


def baidu_download_album(aid, output_dir='.', merge=True, info_only=False):
    html = get_content('http://music.baidu.com/album/{}'.format(aid))
    parser = get_parser(html)
    album_name = parser.find('h2', class_='album-name').text
    artist = parser.find('span', class_='author_list')['title']
    output_dir = '{}/{} - {}'.format(output_dir, artist, album_name)
    ids = json.loads(
        match1(
            html, r'<span class="album-add" data-adddata=\'(.+?)\'>'
        ).replace('&quot', '').replace(';', '"')
    )['ids']
    track_nr = 1
    for _id in ids:
        song_data = baidu_get_song_data(_id)
        song_url = song_data['songLink']
        song_title = song_data['songName']
        song_lrc = song_data['lrcLink']
        file_name = '{:0>2d}.{}'.format(track_nr, song_title)

        _type, ext, size = url_info(song_url, faker=True)
        print_info(site_info, song_title, _type, size)
        if not info_only:
            download_urls(
                [song_url], file_name, ext, size, output_dir, merge=merge,
                faker=True
            )
        if song_lrc:
            _type, ext, size = url_info(song_lrc, faker=True)
            print_info(site_info, song_title, _type, size)
            if not info_only:
                download_urls(
                    [song_lrc], file_name, ext, size, output_dir, faker=True
                )

        track_nr += 1


def baidu_download(
    url, output_dir='.', stream_type=None, merge=True, info_only=False,
    **kwargs
):
    if re.match(r'https?://pan.baidu.com', url):
        real_url, title, ext, size = baidu_pan_download(url)
        print_info('百度网盘', title, ext, size)
        if not info_only:
            print('Hold on...')
            time.sleep(5)
            download_urls(
                [real_url], title, ext, size, output_dir, url, merge=merge,
                faker=True
            )
    elif re.match(r'http://music.baidu.com/album/\d+', url):
        _id = match1(url, r'http://music.baidu.com/album/(\d+)')
        baidu_download_album(_id, output_dir, merge, info_only)

    elif re.match('http://music.baidu.com/song/\d+', url):
        _id = match1(url, r'http://music.baidu.com/song/(\d+)')
        baidu_download_song(_id, output_dir, merge, info_only)

    elif re.match('http://tieba.baidu.com/', url):
        try:
            # embedded videos
            embed_download(url, output_dir, merge=merge, info_only=info_only)
        except Exception:
            # images
            html = get_content(url)
            title = match1(html, r'title:"([^"]+)"')

            items = re.findall(
                r'//imgsrc.baidu.com/forum/w[^"]+/([^/"]+)', html
            )
            urls = [
                'http://imgsrc.baidu.com/forum/pic/item/{}'.format(i)
                for i in set(items)
            ]

            # handle albums
            kw = match1(html, r'kw=([^&]+)') or match1(html, r"kw:'([^']+)'")
            tid = match1(html, r'tid=(\d+)') or match1(html, r"tid:'([^']+)'")
            album_url = (
                'http://tieba.baidu.com/photo/g/bw/picture/list?'
                'kw={}&tid={}&pe={}'.format(kw, tid, 1000)
            )
            album_info = json.loads(get_content(album_url))
            for i in album_info['data']['pic_list']:
                urls.append(
                    'http://imgsrc.baidu.com/forum/pic/item/{}{}'.format(
                        i['pic_id'], '.jpg'
                    )
                )

            ext = 'jpg'
            size = float('Inf')
            print_info(site_info, title, ext, size)

            if not info_only:
                download_urls(
                    urls, title, ext, size, output_dir=output_dir, merge=False
                )


def baidu_pan_download(url):
    errno_patt = r'errno":([^"]+),'
    refer_url = ''
    fake_headers = FAKE_HEADERS.copy()
    fake_headers.update({
        'Host': 'pan.baidu.com',
        'Origin': 'http://pan.baidu.com',
        'Referer': refer_url,
    })
    if cookies:
        print('Use user specified cookies')
    else:
        print('Generating cookies...')
        fake_headers['Cookie'] = baidu_pan_gen_cookies(url)
    refer_url = 'http://pan.baidu.com'
    html = get_content(url, fake_headers, decoded=True)
    isprotected = False
    sign, timestamp, bdstoken, appid, primary_id, fs_id, uk = baidu_pan_parse(
        html
    )
    if sign is None:
        if re.findall(r'verify-property', html):
            isprotected = True
            sign, timestamp, bdstoken, appid, primary_id, fs_id, uk, \
                fake_headers, psk = baidu_pan_protected_share(url)
        if not isprotected:
            raise AssertionError('Share not found or canceled: {}'.format(url))
    if bdstoken is None:
        bdstoken = ''
    if not isprotected:
        sign, timestamp, bdstoken, appid, primary_id, fs_id, \
            uk = baidu_pan_parse(html)
    request_url = (
        'http://pan.baidu.com/api/sharedownload?sign={}&timestamp={}&'
        'bdstoken={}&channel=chunlei&clienttype=0&web=1&app_id={}'.format(
            sign, timestamp, bdstoken, appid
        )
    )
    refer_url = url
    post_data = {
        'encrypt': 0,
        'product': 'share',
        'uk': uk,
        'primaryid': primary_id,
        'fid_list': '[{}]'.format(fs_id)
    }
    if isprotected:
        post_data['sekey'] = psk
    response_content = post_content(request_url, fake_headers, post_data, True)
    errno = match1(response_content, errno_patt)
    if errno != '0':
        raise AssertionError(
            'Server refused to provide download link! (Errno:{})'.format(errno)
        )
    real_url = match1(response_content, r'dlink":"([^"]+)"').replace(
        '\\/', '/'
    )
    title = match1(
        response_content, r'server_filename":"([^"]+)"'
    )
    assert real_url
    _type, ext, size = url_info(real_url, faker=True)
    title_wrapped = json.loads(
        '{{"wrapper":"{}"}}'.format(title)
    )  # \u4ecb\u7ecd -> 介绍
    title = title_wrapped['wrapper']
    return real_url, title, ext, size


def baidu_pan_parse(html):
    sign_patt = r'sign":"([^"]+)"'
    timestamp_patt = r'timestamp":([^"]+),'
    appid_patt = r'app_id":"([^"]+)"'
    bdstoken_patt = r'bdstoken":"([^"]+)"'
    fs_id_patt = r'fs_id":([^"]+),'
    uk_patt = r'uk":([^"]+),'
    # errno_patt = r'errno":([^"]+),'
    primary_id_patt = r'shareid":([^"]+),'
    sign = match1(html, sign_patt)
    timestamp = match1(html, timestamp_patt)
    appid = match1(html, appid_patt)
    bdstoken = match1(html, bdstoken_patt)
    fs_id = match1(html, fs_id_patt)
    uk = match1(html, uk_patt)
    primary_id = match1(html, primary_id_patt)
    return sign, timestamp, bdstoken, appid, primary_id, fs_id, uk


def baidu_pan_gen_cookies(url, post_data=None):
    from http import cookiejar
    cookiejar = cookiejar.CookieJar()
    opener = request.build_opener(request.HTTPCookieProcessor(cookiejar))
    opener.open('http://pan.baidu.com')
    if post_data is not None:
        opener.open(url, bytes(parse.urlencode(post_data), 'utf-8'))
    return cookjar2hdr(cookiejar)


def baidu_pan_protected_share(url):
    print('This share is protected by password!')
    inpwd = input('Please provide unlock password: ')
    inpwd = inpwd.replace(' ', '').replace('\t', '')
    print('Please wait...')
    post_pwd = {
        'pwd': inpwd,
        'vcode': None,
        'vstr': None
    }
    from http import cookiejar
    import time
    cookiejar = cookiejar.CookieJar()
    opener = request.build_opener(request.HTTPCookieProcessor(cookiejar))
    resp = opener.open('http://pan.baidu.com')
    resp = opener.open(url)
    init_url = resp.geturl()
    verify_url = (
        'http://pan.baidu.com/share/verify?{}&t={}&channel=chunlei&'
        'clienttype=0&web=1'.format(
            init_url.split('?', 1)[1], int(time.time())
        )
    )
    refer_url = init_url
    fake_headers = FAKE_HEADERS.copy()
    fake_headers.update({
        'Host': 'pan.baidu.com',
        'Origin': 'http://pan.baidu.com',
        'Referer': refer_url,
    })
    opener.addheaders = dict2triplet(fake_headers)
    pwd_resp = opener.open(
        verify_url, bytes(parse.urlencode(post_pwd), 'utf-8')
    )
    pwd_resp_str = ungzip(pwd_resp.read()).decode('utf-8')
    pwd_res = json.loads(pwd_resp_str)
    if pwd_res['errno'] != 0:
        raise AssertionError(
            'Server returned an error: {} (Incorrect password?)'.format(
                pwd_res['errno']
            )
        )
    pg_resp = opener.open(
        'http://pan.baidu.com/share/link?{}'.format(init_url.split('?', 1)[1])
    )
    content = ungzip(pg_resp.read()).decode('utf-8')
    sign, timestamp, bdstoken, appid, primary_id, fs_id, uk = baidu_pan_parse(
        content
    )
    psk = query_cookiejar(cookiejar, 'BDCLND')
    psk = parse.unquote(psk)
    fake_headers['Cookie'] = cookjar2hdr(cookiejar)
    return (
        sign, timestamp, bdstoken, appid, primary_id, fs_id, uk,
        fake_headers, psk
    )


def cookjar2hdr(cookiejar):
    cookie_str = ''
    for i in cookiejar:
        cookie_str = cookie_str + i.name + '=' + i.value + ';'
    return cookie_str[:-1]


def query_cookiejar(cookiejar, name):
    for i in cookiejar:
        if i.name == name:
            return i.value


def dict2triplet(dictin):
    out_triplet = []
    for i in dictin:
        out_triplet.append((i, dictin[i]))
    return out_triplet


download = baidu_download
download_playlist = playlist_not_supported(site_info)
